/* Copyright 2013 Webalon Ltd
 *
 * It is strictly forbidden to copy, distribute, modify or reuse this file or its
 * contents without written permission from the copyright holder, Webalon Ltd
 *
 **************************************************************************************/

var AJKButtonPopupController = function(data) {
	this.domButtons = data.domButtons;
	this.buttonPopups = new Array();
	this.buttonPopupsByKey = new Array();
	this.selectedPopup = "";
	this.observers = new Array();
	this.adjustFunc = (data.adjustFunc) ? data.adjustFunc : function() { return 0; } 
}

AJKButtonPopupController.prototype = {
	init: function() {
		var self = this;
		$(self.domButtons).each( function() {
			var popupId = $(this).attr("popupId");
			
			if (popupId) {
				var aButtonPopup = new AJKButtonPopup({
					domRootEl: $("#"+popupId).get()[0],
					domButton: this,
					controller: self,
					id: popupId
				}).init();
				
				self.buttonPopups.push(aButtonPopup);
				self.buttonPopupsByKey[popupId] = aButtonPopup;
	
	
				$(this).click( function() {
					if (self.selectedPopup == aButtonPopup) {
						aButtonPopup.hide();
						self.popupDidClose();
					}
					else {
						aButtonPopup.display();
						if (self.selectedPopup) {
							self.selectedPopup.hide();
							self.popupDidClose();
						}
						self.selectedPopup = aButtonPopup;
						self.popupDidOpen(); 
					}
					return false;
				});
			}
		});
		
		return self;	
	},
	popupDidOpen: function() {
		var self = this;
		if (self.observers[self.selectedPopup.id]) {
			$.each(self.observers[self.selectedPopup.id], function() {
				if (this.buttonPopupDidOpen) {
					this.buttonPopupDidOpen({
						popup: self.selectedPopup
					});
				}
			});	
		}
		theAJKKeyEvent.registerAsObserver({
			observer: self
		});
	},
	popupDidClose: function() {
		var self = this;
		if (self.observers[self.selectedPopup.id]) {
			$.each(self.observers[self.selectedPopup.id], function() {
				if (this.buttonPopupDidClose) {
					this.buttonPopupDidClose({
						popup: self.selectedPopup
					});
				}
			});	
		}
		self.selectedPopup = ""; 
		theAJKKeyEvent.removeAsObserver({
			observer: self
		});
	},
	registerAsObserverOfType: function(dataObj) {
		var self = this;
		var type = dataObj.type;
		var observer = dataObj.observer;
		if (!self.observers[type]) {
			self.observers[type] = new Array();
		}
		self.observers[type].push(observer);
	},
	keyEventTookPlace: function(dataObj) {
		var self = this;
		var key = dataObj.key;
		if (key == 27 && self.selectedPopup) {
			self.selectedPopup.hide();	
			self.popupDidClose();
		}	
	},
	forceOpenPopupOfType: function(dataObj) {
		var self = this;		
		var type = dataObj.type;
		if (!self.buttonPopupsByKey[type].open) {
			$(self.buttonPopupsByKey[type].domButton).click();
		}
	},
	forceClosePopupOfType: function(dataObj) {
		var self = this;		
		var type = dataObj.type;
		if (self.buttonPopupsByKey[type].open) {
			$(self.buttonPopupsByKey[type].domButton).click();
		}
	}

}

var AJKButtonPopup = function(dataObj) {
	this.domRootEl = dataObj.domRootEl;
	this.domButton = dataObj.domButton;
	this.id = dataObj.id;
	this.domCloseButtons = $(this.domRootEl).find("a.close").get();
	this.observers = new Array();
	this.open = false;
	this.controller = dataObj.controller;
	this.animateStartTop = -100; 
	this.animateEndTop = 40; 
}

AJKButtonPopup.prototype = {
	init: function() {
		var self = this;
		$(self.domCloseButtons).click( function() {
			self.hide();
			self.controller.popupDidClose();
			return false;
		});
		return self;	
	},
	display: function() {
		var self = this;
		
		var rightOffset = theAJKWindowResizeEvent.lastWindowSize.width - AJKHelpers.getCoordsOfDomEl({ domEl: self.domButton }).x + self.controller.adjustFunc();
		rightOffset -= parseInt($(self.domButton).parent().width()/2);
		
		
		if ($.browser.msie && $.browser.version < 9) {
			$(self.domRootEl)
			.css({top: self.animateStartTop, display:"block", right: rightOffset })
			.animate({
				top: self.animateEndTop
			},250, function() {
				//	
			});
		}
		else {
			$(self.domRootEl)
			.css({opacity: 0, top: self.animateStartTop, display:"block", right: rightOffset })
			.animate({
				top: self.animateEndTop,
				opacity: 1
			},250, function() {
				//	
			});
		}
		self.open = true;
	},
	hide: function() {
		var self = this;

		if ($.browser.msie && $.browser.version < 9) {
			$(self.domRootEl)
			.animate({
				top: self.animateStartTop
			},250, function() {
				$(this).css({display:"none"});	
			});
		}
		else {
			$(self.domRootEl)
			.animate({
				top: self.animateStartTop,
				opacity: 0
			},250, function() {
				$(this).css({display:"none"});	
			});
		}
		self.open = false;
	}
}