/* Copyright 2013 Webalon Ltd
 *
 * It is strictly forbidden to copy, distribute, modify or reuse this file or its
 * contents without written permission from the copyright holder, Webalon Ltd
 *
 **************************************************************************************/

/* Video audio panel */


var AJKVideoSelector = function(data) {
	var self = this;	
	self.controller = data.controller;
	self.mainController = data.controller.mainController;
	self.timeline = data.timeline;
	self.domFeedItemSelector = data.domFeedItemSelector,
	self.initialised = false;
	self.domMenu = "";
	self.selectors = [];
	self.selectedSelectorType = "upload";
	self.domContentField = "";
	self.selectedPanel = "";
}

AJKVideoSelector.prototype = {
	init: function() {
		var self = this;
		
		return self;	
	},
	initialise: function() {
		var self = this;
		self.domMenu = $('<div class="ft-ah-image-selector-tabs"><ul><li class="upload selected" option="upload">Local video/audio file<span><li option="url">Enter video/audio url<span></span></li></span></li></ul></div>').get()[0];

		$(self.domMenu).find("li").click( function() {
			var selectorType = $(this).attr("option");
			if (selectorType != self.selectedSelectorType) {
				$(self.domMenu).find(".selected").removeClass("selected");
				$(this).addClass("selected");
				self.selectors[self.selectedSelectorType].close();
				self.launchForSelectorType({ selectorType: selectorType }); 
			}
			return false;
		});
		
		self.initialised = true;
	},
	initialiseSelector: function(data) {
		var self = this;
		var selectorType = data.selectorType;
		
		if (selectorType == "upload") {
			self.selectors[selectorType] = new AJKUploadVideoSelector({
				videoSelectedFunc: function(data) {
					self.selectVideo(data);
				}
			}).init();
			
			AJKUploadVideoSelector.selector1 = self.selectors[selectorType];
		}
		else if (selectorType == "url") {
			self.selectors[selectorType] = new AJKVideoUrlSelector({
				videoSelectedFunc: function(data) {
					self.selectVideo(data);
				}
			}).init();
		}
	},
	selectVideo: function(data) {
		var self = this;
		var videoSrc = data.src;
		if (self.domThumbPosField) {
			$(self.domThumbPosField).val("0,0").blur();	
		}
		$(self.domContentField).val(videoSrc);
		$(self.domContentField).blur();
		//self.showSelectedVideoPanelForVideo({ src: videoSrc });
	},
	injectMenuInto: function(data) {
		var self= this;
		var selector = data.selector;
		$(selector.domRoot).find(".ajk-fis-panel").prepend(self.domMenu);
	},
	launch: function(data) {
		var self = this;
		if (!self.initialised) {
			self.initialise();	
		}

		self.domContentField = data.domContentField;
		self.domThumbPosField = data.domThumbPosField;
		
		
		self.launchForSelectorType({
			selectorType: self.selectedSelectorType
		});
	},
	launchSelector: function(data) {
		var self = this;
		var selector = data.selector;
		self.injectMenuInto({ selector:  selector });

		selector.launch({
			domContentField: self.domContentField,
			domThumbPosField: self.domThumbPosField,
		});
	},
	launchForSelectorType: function(data) {
		var self = this;
		self.selectedSelectorType = data.selectorType;
		
		if (!self.selectors[self.selectedSelectorType]) {
			self.initialiseSelector({ selectorType: self.selectedSelectorType });
		}
		self.launchSelector({ selector: self.selectors[self.selectedSelectorType]});
	}
}

var AJKVideoUrlSelector = function(data) {
	var self = this;
	self.domRoot = "";
	self.lightbox = "";
	self.videoSelectedFunc = data.videoSelectedFunc;
	self.domContent = $("#ft-ah-imglink-panel").clone().get()[0];
	self.domInput = $(self.domContent).find("input").get()[0];
	self.domButton = $(self.domContent).find(".button").get()[0];
	self.domImage = $(self.domContent).find("img").get()[0];
}

AJKVideoUrlSelector.prototype = {
	init: function() {
		var self = this;
		$(self.domContent).attr("id","");
		self.lightbox = new TLAdminLightbox({
			domClass: "tl-ah-url-image-lightbox tl-ah-url-video-lightbox",
			title: TLConfigText['videoURLSelect_title'],
			intro: TLConfigText['videoURLSelect_intro'],
			domContent: self.domContent
		}).init();	
		
		self.domRoot = self.lightbox.domRoot;
		
		$(self.domButton).click( function() {
			var videoSrc = self.processUrl({ url: $(self.domInput).val() });
			self.videoSelectedFunc({ src: videoSrc });
			return false;
		});
 		return self;		
	},
	processUrl: function(data) {
		var self = this;
		return data.url;
	},
	launch: function(data) {
		var self = this;
		var domContentField = data.domContentField;
		var curVideo = $(domContentField).val();
		$(self.domInput).val(curVideo);
        self.domImage.src = AJKHelpers.emptyImage;
		self.lightbox.openPanel();
	},
	close: function() {
		var self = this;
		self.lightbox.closePanel();	
	}
}

var AJKUploadVideoSelector = function(data) {
	var self = this;
	self.domRoot = "";
	self.lightbox = "";
	self.videoSelectedFunc = data.videoSelectedFunc;
	self.lastEventType = "";
}

AJKUploadVideoSelector.prototype = {
	init: function() {
		var self = this;
		var domContent = $("#ft-ah-imgup-upload").clone().get()[0];
		$(domContent).attr("id","");
		
        self.domUploadButton = $(domContent).find(".ft-ah-imgup-upload-button").get()[0];

		self.lightbox = new TLAdminLightbox({
			domClass: "tl-ah-upload-image-lightbox tl-ah-upload-video-lightbox",
			title: TLConfigText['videoUploader_title'],
			intro: TLConfigText['videoUploader_intro'],
			domContent: domContent
		}).init();	
		
		self.domRoot = self.lightbox.domRoot;
		
		$(self.domUploadButton).click( function() {
			// Here, we need to get objC to open the panel for an image
			theTLParantWindow.NSDocument.loadVideoFromDisk();
			return false;	
		});
		
		$(self.domUploadPanel).find("a.cancel").click( function() {
			self.clearStateStyles();
			return false;
		});
		
 		return self;		
	},
	userSelectedVideo: function(data) {
		var self = this;	
		var videoPath = new String(data.videoPath); // Needed to convert Cocoa string to JS String!!!
		self.close();
		self.videoSelectedFunc({ src: videoPath });
		self.clearStateStyles();
	},
	clearStateStyles: function() {
		var self = this;
	},
	launch: function(data) {
		var self = this;
		self.clearStateStyles();
		self.lightbox.openPanel();
	},
	close: function() {
		var self = this;
		self.lightbox.closePanel();	
	}
}

/* Image panel */

var AJKImageSelector = function(data) {
	var self = this;
	self.controller = data.controller;
	self.mainController = data.controller.mainController;
	self.timeline = data.timeline;
	self.domFeedItemSelector = data.domFeedItemSelector,
	self.flickrImageSelector = "";
	self.initialised = false;
	self.domMenu = "";
	self.selectors = [];
	self.selectedSelectorType = "upload";
	self.domContentField = "";
	self.domCaptionField = "";
	self.selectedImagePanel = "";
	self.isStandardPage = data.isStandardPage;
}

AJKImageSelector.prototype = {
	init: function() {
		var self = this;
		return self;	
	},
	initialise: function() {
		var self = this;
		self.domMenu = $('<div class="ft-ah-image-selector-tabs"><ul><li class="upload selected" option="upload">Local image<span><li option="url">Enter image url<span></span></li></span></li><li class="ft-ah-ist-flickr" option="flickr">Flickr image<span></span></li></ul></div>').get()[0];

		if (self.isStandardPage) {
			$(self.domMenu).find(".upload").css({ display: "none" });
		}

		$(self.domMenu).find("li").click( function() {
			var selectorType = $(this).attr("option");
			if (selectorType != self.selectedSelectorType) {
				$(self.domMenu).find(".selected").removeClass("selected");
				$(this).addClass("selected");
				self.selectors[self.selectedSelectorType].close();
				self.launchForSelectorType({ selectorType: selectorType }); 
			}
			return false;
		});
		
		self.initialised = true;
	},
	imagesLeftForTerm: function(data) {
		var self = this;
		var term = data.term; 
		var numImagesLeft = data.maxImages;

		if (self.timeline.backgroundImage && self.timeline.backgroundImage.indexOf(term) != -1) {
			numImagesLeft--;
		}
		if (self.timeline.introImage && self.timeline.introImage.indexOf(term) != -1) {
			numImagesLeft--;
		}
		
		$.each( self.mainController.markers, function() {
			$.each( this.images, function() {
				if (this.src && this.src.indexOf(term) != -1) {
					numImagesLeft--;
				}
			});
		});	
		return ((numImagesLeft < 0) ? 0 : numImagesLeft);
	},
	initialiseSelector: function(data) {
		var self = this;
		var selectorType = data.selectorType;
		
		if (selectorType == "flickr") {
			self.selectors[selectorType] = new AJKFlickrImageSelector({
				domRoot: self.domFeedItemSelector,
				controller: self,
				imageSelectedFunc: function(data) {
					self.selectImage(data);
				},
				remainingFlickrImagesFunc: function() {
					return self.imagesLeftForTerm({
						term: "flickr.com",
						maxImages: 30	
					});
				}
			}).init();
		}
		else if (selectorType == "upload") {
			self.selectors[selectorType] = new AJKUploadImageSelector({
				imageSelectedFunc: function(data) {
					self.selectImage(data);
				},
				imageLimitFunc: function() {
					return self.numUploadedImagesAllowed();
				},
				remainingImagesFunc: function() {
					return self.imagesLeftForTerm({
						term: AJKHelpers.uploadDomain,
						maxImages: self.numUploadedImagesAllowed()	
					});
				}
			}).init();
			
			AJKUploadImageSelector.selector1 = self.selectors[selectorType];
		}
		else if (selectorType == "url") {
			self.selectors[selectorType] = new AJKImageUrlSelector({
				imageSelectedFunc: function(data) {
					self.selectImage(data);
				}
			}).init();
		}
	},
	numUploadedImagesAllowed: function() {
		var self = this;
		var account = self.controller.timeline.accountType;
		var numImages = 99999; 
		return numImages;
	},
	selectImage: function(data) {
		var self = this;
		var imageSrc = data.src;
		var dataUri = (data.dataUri) ? data.dataUri : "";
		if (self.domThumbPosField) {
			$(self.domThumbPosField).val("0,0").blur();	
		}
		$(self.domContentField).val(imageSrc);
		$(self.domContentField).blur();
		if (self.domDataUriField) { // Force update of the data uri 
			$.data(self.domDataUriField, "value",dataUri);
			$(self.domDataUriField).blur();
		}
		var displayImageSrc = (dataUri) ? dataUri : imageSrc; 
		self.showSelectedImagePanelForImage({ src: displayImageSrc });
	},
	injectMenuInto: function(data) {
		var self= this;
		var selector = data.selector;
		$(selector.domRoot).find(".ajk-fis-panel").prepend(self.domMenu);
	},
	launch: function(data) {
		var self = this;
		if (!self.initialised) {
			self.initialise();	
		}

		self.imageSize = data.imageSize;
		self.domContentField = data.domContentField;
		self.domCaptionField = data.domCaptionField;
		self.domThumbPosField = data.domThumbPosField;
		self.domDataUriField = data.domDataUriField;
		
		
		var srcValue = $(self.domContentField).val();
		var dataUriVal = $.data(self.domDataUriField, "value");
		
		if (dataUriVal) {
			self.showSelectedImagePanelForImage({ src: dataUriVal });
		}
		else if (srcValue && srcValue != TLConfigText['marker_media_Enter_meda_source_here']) {
			self.showSelectedImagePanelForImage({ src: srcValue });
		}
		else {
			self.launchForSelectorType({
				selectorType: self.selectedSelectorType
			});
		}
	},
	initialiseSelectedImagePanel: function() {
		var self = this;
		self.selectedImagePanel = new AJKSelectedImagePanel({
			newImageFunc: function() {
				self.launchForSelectorType({selectorType: self.selectedSelectorType});
			},
			positionImageFunc: function() {
				
			}	
		}).init();
		
		var domLaunchButton = $(self.selectedImagePanel.domContent).find("#tl-ah-image-position-panel-launch").get()[0];
		$(domLaunchButton).click( function() {
				self.selectedImagePanel.close();
		});
		
		if (!self.isStandardPage) { // Only initialise if on admin pahe, not homelogged in
			new TLImagePositioningController({
				imageSelector: self,
				domLaunchButton: domLaunchButton,
				positioningFinishedFunc: function() {
					self.selectedImagePanel.open();
				}
			}).init();
		}	
	},
	showSelectedImagePanelForImage: function(data) {
		var self = this;
		var imageSrc = data.src;
		var selector = self.selectors[self.selectedSelectorType];
		if (selector) {
			selector.close();
		}
		if (!self.selectedImagePanel) {
			self.initialiseSelectedImagePanel();
		}
		self.selectedImagePanel.openPanelWithImage({ 
			src: imageSrc,
			positionable: self.domThumbPosField
		 });
	},
	launchSelector: function(data) {
		var self = this;
		var selector = data.selector;
		self.injectMenuInto({ selector:  selector });

		selector.launch({
			imageSize: self.imageSize,
			domContentField: self.domContentField,
			domCaptionField: self.domCaptionField,
			domThumbPosField: self.domThumbPosField,
			domDataUriField: self.domDataUriField
		});
	},
	launchForSelectorType: function(data) {
		var self = this;
		self.selectedSelectorType = data.selectorType;
		
		if (!self.selectors[self.selectedSelectorType]) {
			self.initialiseSelector({ selectorType: self.selectedSelectorType });
		}
		self.launchSelector({ selector: self.selectors[self.selectedSelectorType]});
	}
}

var AJKSelectedImagePanel = function(data) {
	var self = this;
	self.positionImageFunc = data.positionImageFunc;
	self.newImageFunc = data.newImageFunc;
	self.lightbox = "";
	self.domContent = "";
	self.domImageHolder = "";
	self.showPosClass = "tl-ah-fis-show-positioning";
}

AJKSelectedImagePanel.prototype = {
	init: function() {
		var self = this;
		self.initialiseContent();
		self.lightbox = new TLAdminLightbox({
			domClass: "tl-ah-selected-image-lightbox",
			title: "",
			intro: "",
			domContent: self.domContent
		}).init();	

		return self;
	},
	initialiseContent: function() {
		var self = this;
		
		var insHTML = '<div><div class="tl-ah-fis-selected-image-holder ajk-fis-item-selected-image-holder">';
		insHTML += '<div class="tl-ah-fis-selected-image ajk-fis-item-selected-image"></div>';
		insHTML += '<div class="ajk-fis-selected-message tl-fis-selected-message">';
		insHTML += '<p>You have this image selected. Close panel or choose <a class="tl-ah-fis-new-image-link" href="#">new image</a></p>';
		insHTML += '</div>';
		insHTML += '</div>';
		insHTML += '<a id="tl-ah-image-position-panel-launch" class="tl-ah-field-option" href="#">Image positioning</a></div>';

		self.domContent = $(insHTML).get()[0];
		self.domImageHolder = $(self.domContent).find(".tl-ah-fis-selected-image").get()[0];
		
		$(self.domContent).find(".tl-ah-fis-new-image-link").click( function() {
			self.close();
			self.newImageFunc();
			return false;
		});
		
	},
	openPanelWithImage: function(data) {
		var self = this;
		var src = data.src;
		var positionable = data.positionable;
		$(self.domImageHolder).empty().append('<img src="'+src+'" />');

		if (positionable) {
			$(self.domContent).addClass(self.showPosClass);
		}
		else {
			$(self.domContent).removeClass(self.showPosClass);
		}

		self.lightbox.openPanel();
	},
	open: function() {
		var self = this;
		self.lightbox.openPanel();
	},
	close: function() {
		var self = this;
		self.lightbox.closePanel();	
	}
}

var AJKImageUrlSelector = function(data) {
	var self = this;
	self.domRoot = "";
	self.lightbox = "";
	self.imageSelectedFunc = data.imageSelectedFunc;
	self.domContent = $("#ft-ah-imglink-panel").clone().get()[0];
	self.domInput = $(self.domContent).find("input").get()[0];
	self.domButton = $(self.domContent).find(".button").get()[0];
	self.domImage = $(self.domContent).find("img").get()[0];
}

AJKImageUrlSelector.prototype = {
	init: function() {
		var self = this;
		$(self.domContent).attr("id","");
		
		self.lightbox = new TLAdminLightbox({
			domClass: "tl-ah-url-image-lightbox",
			title: TLConfigText['imageURLSelect_title'],
			intro: TLConfigText['imageURLSelect_intro'],
			domContent: self.domContent
		}).init();	
		
		self.domRoot = self.lightbox.domRoot;
		
		$(self.domButton).click( function() {
			var imgSrc = self.processUrl({ url: $(self.domInput).val() });
			self.imageSelectedFunc({ src: imgSrc });
			return false;
		});
 		return self;		
	},
	processUrl: function(data) {
		var self = this;
		var url = data.url;
		if (url && url.indexOf("http://") == -1 && url.indexOf("https://") == -1)  {
			url = "http://"+url;
		}
		return url;
	},
	launch: function(data) {
		var self = this;
		var domContentField = data.domContentField;
		var curImage = $(domContentField).val();
		$(self.domInput).val(curImage);
		if (curImage && (curImage.indexOf("http://") != -1 || curImage.indexOf("https://") != -1))  {
			self.domImage.src = curImage;
		}
		else {
			self.domImage.src = AJKHelpers.emptyImage;
		}
		self.lightbox.openPanel();
	},
	close: function() {
		var self = this;
		self.lightbox.closePanel();	
	}
}

var AJKUploadImageSelector = function(data) {
	var self = this;
	self.domRoot = "";
	self.lightbox = "";
	self.remainingImagesFunc = data.remainingImagesFunc;
	self.imageLimitFunc = data.imageLimitFunc;
	self.imageSelectedFunc = data.imageSelectedFunc;
	self.lastEventType = "";
}

AJKUploadImageSelector.prototype = {
	init: function() {
		var self = this;
		var domContent = $("#ft-ah-imgup-upload").clone().get()[0];
		$(domContent).attr("id","");
        self.domUploadButton = $(domContent).find(".ft-ah-imgup-upload-button").get()[0];
		
		self.lightbox = new TLAdminLightbox({
			domClass: "tl-ah-upload-image-lightbox",
			title: TLConfigText['imageUploader_title'],
			intro: TLConfigText['imageUploader_intro'],
			domContent: domContent
		}).init();	
		
		self.domRoot = self.lightbox.domRoot;
		
		self.domRemainingImages = $("#tl-image-uploader-remaining").get()[0]; 
		self.domImageLimit = $("#tl-image-uploader-limit").get()[0]; 

		$(self.domUploadButton).click( function() {
			// Here, we need to get objC to open the panel for an image
			theTLParantWindow.NSDocument.loadImageFromDisk();
			return false;	
		});
		 
 		return self;		
	},
	userSelectedImage: function(data) {
		var self = this;	
		var imagePath = new String(data.imagePath); // Needed to convert Cocoa string to JS String!!!
		var maxSize = (self.imageSize == "large") ? { width: 1500, height: 1500 } : { width: 800, height: 800 };
		new AJKImageLoader({
			imageUrl: imagePath,
			loadCallback: function(data) {
				var loadedImage = data.image
				var base64Url = AJKHelpers.getDataUrlForImage({
					image: loadedImage,
					maxSize: maxSize
				});
				self.close();
				var imageNameArr = imagePath.split("/");
				var imageName = (imageNameArr) ? imageNameArr[imageNameArr.length-1] : imagePath;
				self.imageSelectedFunc({ src: imageName, dataUri: base64Url });
				self.clearStateStyles();
			}
		}).init();
	},
	clearStateStyles: function() {
		var self = this;
	},
	updateRemainingImages: function() {
		var self = this;
		
		if (self.remainingImagesFunc && !self.isStandardPage) {
			var numImages = self.remainingImagesFunc();
			$(self.domRemainingImages).text(numImages); 
			if (numImages > 0) {
				$(self.domRoot).removeClass("tl-ah-fis-disable-image-selection");
			}
			else {
				$(self.domRoot).addClass("tl-ah-fis-disable-image-selection");
			}
		}

		if (self.imageLimitFunc) {
			var numImages = self.imageLimitFunc();
			$(self.domImageLimit).text(numImages); 
		}
	},
	launch: function(data) {
		var self = this;
		self.imageSize = data.imageSize;
		self.clearStateStyles();
		self.updateRemainingImages();
		self.lightbox.openPanel();
	},
	close: function() {
		var self = this;
		self.lightbox.closePanel();	
	}
}

var AJKFlickrImageSelector = function(data) {
	var self = this;
	self.domRoot = data.domRoot;
	self.controller = data.controller;
	self.imageSelectedFunc = data.imageSelectedFunc;
	self.remainingFlickrImagesFunc = data.remainingFlickrImagesFunc;
	self.feedSelector = "";	
	self.flickrApiKey = theTLSettings.flickrApiKey; 
	self.numItemsPerPage = 12;
	self.imageSize = "";
	self.cachedFlickrUsers = new Array();
	self.domNumImages = "";
	self.domCaptionField = "";
}

AJKFlickrImageSelector.prototype = {
	init: function() {
		var self = this;
		
		/* setTimeout( function() {
			$(".tl-ah-search-button:eq(0)").click();
		},500); */

		self.feedSelector = new AJKFeedItemSelector({
			domRoot: self.domRoot,
			domClass: "tl-ah-flickr-image-selector",
			title: TLConfigText['flickrImageSelector_title'],
			intro: TLConfigText['flickrImageSelector_intro'],
			numItemsPerPage: self.numItemsPerPage,
			option1Title: TLConfigText['flickrImageSelector_option1Title'],
			option1Text: TLConfigText['flickrImageSelector_option1Text'] ,
			option1Label: TLConfigText['flickrImageSelector_option1Label'],
			option1Message: TLConfigText['flickrImageSelector_option1Message']+" 'X_CRITERIA_X'",
			option2Title: TLConfigText['flickrImageSelector_option2Title'],
			option2Text: TLConfigText['flickrImageSelector_option2Text'],
			option2Label:TLConfigText['flickrImageSelector_option2Label'],
			option2Message: TLConfigText['flickrImageSelector_option2Message']+" 'X_CRITERIA_X'",
			itemSelectedFunc: function(data) {
				var item = data.item;
				self.showSelectedItem({ src: item[self.imageSize] });
				if (self.domCaptionField) {
					// We also need to generate the caption
					$.getJSON("http://api.flickr.com/services/rest/?jsoncallback=?", {
						method: "flickr.photos.getInfo",
						api_key: self.flickrApiKey,
						photo_id: item.id,
						secret: item.secret,
						format: "json"
					}, function(data) {
						var displayName = (data.photo.owner.realname) ? data.photo.owner.realname : data.photo.owner.username;
						var userUrl = "http://www.flickr.com/people/"+data.photo.owner.nsid+"/";
						var caption = TLConfigText['flickrImageSelector_Photo_credit']+': <a href="'+userUrl+'">'+displayName+'</a>';
						$(self.domCaptionField).val(caption)
						$(self.domCaptionField).blur();
						if (self.domThumbPosField) {
							$(self.domThumbPosField).val("0,0").blur();	
						}
						if (self.domDataUriField) { // Force update of the 
							$.data(self.domDataUriField, "value","");
							$(self.domDataUriField).blur();
						}

					});
				}
			},
			processFeedFunc: function(data) {
				var fieldData = data.fieldData;
				var page = data.page;
				var callback = data.callback;
				if (fieldData["opt1Search"]) {
					// General search
					$.getJSON("http://api.flickr.com/services/rest/?jsoncallback=?", {
						method: "flickr.photos.search",
						text: fieldData["opt1Search"],
						page: data.page,
						per_page: self.numItemsPerPage,
						license: "1,2,3,4,5,6,7",
						api_key: self.flickrApiKey,
						format: "json"
					}, function(data) {
						var numPages = data.photos.pages;
						
						self.feedSelector.setNumberOfPages({ value: numPages });
						
						var returnObjs = self.processImageJSONFromArray({
							anArray: data.photos.photo	
						});
						
						if (callback) {
							callback({
								itemObjs: returnObjs
							});	
						}
					});
					
				}
				else if (fieldData["opt2Search"]) {
					var flickrUsername = fieldData["opt2Search"];
					 
					if (!self.cachedFlickrUsers[flickrUsername]) {
						$.getJSON("http://api.flickr.com/services/rest/?jsoncallback=?", {
							method: "flickr.people.findByUsername",
							username: flickrUsername,
							api_key: self.flickrApiKey,
							format: "json"
						}, function(data) {
							if (!data.user) {
								if (callback) {
									callback({});	
								}
								return;	
							}
							
							var userId = data.user.nsid;
							self.cachedFlickrUsers[flickrUsername] = {
								username: flickrUsername,
								userId: userId	
							}
							
							self.getPhotosForUser({
								userId: userId,
								page: page,
								callback: function(data) {
									if (callback) {
										callback(data);	
									}
								}
							});
						});
					}
					else {
						self.getPhotosForUser({
							userId: self.cachedFlickrUsers[flickrUsername].userId,
							page: page,
							callback: function(data) {
								if (callback) {
									callback(data);	
								}
							}
						});
					}
					
				}
			}
		}).init();
		

		self.domNumImages = $(self.domRoot).find("#tl-flickr-image-limiter").get()[0];

		return self;	
	},
	showSelectedItem: function(data) {
		var self = this;
		var src = data.src;
		self.imageSelectedFunc({ src: src });
	},
	getPhotosForUser: function(data) {
		var self = this;
		var userId = data.userId;
		var callback = data.callback;
		var page = data.page;
		$.getJSON("http://api.flickr.com/services/rest/?jsoncallback=?", {
			method: "flickr.people.getPublicPhotos",
			user_id: userId,
			page: page,
			per_page: self.numItemsPerPage,
			api_key: self.flickrApiKey,
			format: "json"
		}, function(data) {
			var numPages = data.photos.pages;
			self.feedSelector.setNumberOfPages({ value: numPages });
			var returnObjs = self.processImageJSONFromArray({
				anArray: data.photos.photo	
			});
			
			if (callback) {
				callback({
					itemObjs: returnObjs
				});	
			}
		});
		
	},
	processImageJSONFromArray: function(data) {
		var self = this;
		var anArray = data.anArray;
		var returnObjs = new Array();
		var counter = 0;
		$.each(anArray, function() {
			var anObj = {
				id: this.id,
				owner: this.owner,
				caption: this.title,
				secret: this.secret,
				thumb: "http://farm"+this.farm+".static.flickr.com/"+this.server+"/"+this.id+"_"+this.secret+"_s.jpg",
				small: "http://farm"+this.farm+".static.flickr.com/"+this.server+"/"+this.id+"_"+this.secret+"_m.jpg",
				medium: "http://farm"+this.farm+".static.flickr.com/"+this.server+"/"+this.id+"_"+this.secret+"_-.jpg",
				large: "http://farm"+this.farm+".static.flickr.com/"+this.server+"/"+this.id+"_"+this.secret+"_b.jpg",
				index: counter++
			}
			
			anObj.domEl = self.createDomItem({ anItemObj: anObj });
			
			returnObjs.push(anObj);
		});		
		return returnObjs; 
	},
	createDomItem: function(data) {
		var self = this;
		var anItemObj = data.anItemObj;
		var thumbUrl = anItemObj.thumb;
		var index = anItemObj.index;
		var extraClass = (index % (self.numItemsPerPage/2) == 0) ? " tl-ah-fis-image-block-row-start" : "";

		var insertHTML = "";
		insertHTML+='<div class="tl-ah-fis-image-block'+extraClass+'">';
		insertHTML+='<a class="ajk-fis-select-click tl-ah-fis-image-holder" href="#">';
		insertHTML+='<img src="'+thumbUrl+'"/>';
		insertHTML+='</a>';
		insertHTML+='<a class="tl-ah-fis-image-expand ajk-fis-image-expand" href="#">'+TLConfigText['flickrImageSelector_Expand']+'</a>';
		insertHTML+='</div>';
		return $(insertHTML).get()[0];
	},
	updateRemainingImages: function() {
		var self = this;
		if (self.remainingFlickrImagesFunc && !self.controller.isStandardPage) {
			var numImages = self.remainingFlickrImagesFunc();
			$(self.domNumImages).text(numImages); 
			if (numImages > 0) {
				$(self.domRoot).removeClass("tl-ah-fis-disable-image-selection");
			}
			else {
				$(self.domRoot).addClass("tl-ah-fis-disable-image-selection");
			}
		}
	},
	launch: function(data) {
		var self = this;

		self.imageSize = data.imageSize;
		self.domCaptionField = data.domCaptionField;
		self.domThumbPosField = data.domThumbPosField;
		self.domDataUriField = data.domDataUriField;

		if (self.alreadyLaunched) {
			return self.open();
		}
		self.alreadyLaunched = true;
		self.updateRemainingImages();
		self.feedSelector.openPanel();
	},
	close: function() {
		var self = this;
		self.updateRemainingImages();
		self.feedSelector.closePanel();	
	},
	open: function() {
		var self = this;	
		self.feedSelector.openPanel();	
	}
}

var AJKFeedItemSelector = function(data) {
	var self = this;
	self.domRoot = data.domRoot;
	self.domFade = $(self.domRoot).find(".ajk-ah-fis-fade").get()[0];
	self.domMainTitle = $(self.domRoot).find(".ajk-fis-header-title").get()[0];
	self.domMainIntro = $(self.domRoot).find(".ajk-fis-header-intro").get()[0];
	self.domOption1Title = $(self.domRoot).find(".ajk-fis-option-1-title").get()[0];
	self.domOption1Text = $(self.domRoot).find(".ajk-fis-option-1-text").get()[0];
	self.domOption2Title = $(self.domRoot).find(".ajk-fis-option-2-title").get()[0];
	self.domOption2Text = $(self.domRoot).find(".ajk-fis-option-2-text").get()[0];
	self.domOption1Input = $(self.domRoot).find(".ajk-fis-option-1-input").get()[0];
	self.domOption2Input = $(self.domRoot).find(".ajk-fis-option-2-input").get()[0];
	self.domItemCarouselStage = $(self.domRoot).find(".ajk-fis-item-carousel-stage").get()[0];
	self.domItemCarouselFade = $(self.domRoot).find(".ajk-fis-item-carousel-fade").get()[0];
	self.domNumPages = $(self.domRoot).find(".ajk-fis-controls-num-pages").get()[0];
	self.domCurrentPage = $(self.domRoot).find(".ajk-fis-controls-page").get()[0];
	
	self.domPanel = $(self.domRoot).find(".ajk-fis-panel").get()[0];
	
	self.domClose = $(self.domRoot).find(".ajk-fis-close").get()[0];
	self.domNextPage = $(self.domRoot).find(".ajk-fis-controls-next").get()[0];
	self.domPrevPage = $(self.domRoot).find(".ajk-fis-controls-prev").get()[0];
	self.domControls = $(self.domRoot).find(".ajk-fis-controls").get()[0];
	self.domSearchMessage = $(self.domRoot).find(".ajk-fis-search-text").get()[0]; 
	
	self.itemSelectedFunc = data.itemSelectedFunc;
	
	self.domClass = data.domClass;
	self.title = data.title;
	self.intro = data.intro;
	self.option1Title = data.option1Title;
	self.option1Text = data.option1Text;
	self.option1Label = data.option1Label;
	self.option1Message = data.option1Message;
	self.option2Title = data.option2Title;
	self.option2Text = data.option2Text;
	self.option2Label = data.option2Label;
	self.option2Message = data.option2Message;
	self.processFeedFunc = data.processFeedFunc;
	self.createDomItem = data.createDomItem;
	self.numItemsPerPage = data.numItemsPerPage;
	self.currentlySubmitting = false;
	
	self.currentPage = 1;
	self.numItemsPerPage = data.numItemsPerPage;
	self.numPages = 0;
	
	self.selectedVerifier = "";
	self.activeFieldData = "";
	self.panelHeight = "";
}


AJKFeedItemSelector.prototype = {
	init: function() {
		var self = this;	
		$(self.domRoot).addClass(self.domClass);
		$(self.domMainTitle).html(self.title);
		$(self.domMainIntro).html(self.intro);
		$(self.domOption1Title).html(self.option1Title);
		$(self.domOption1Text).html(self.option1Text);
		$(self.domOption2Title).html(self.option2Title);
		$(self.domOption2Text).html(self.option2Text);
		
		$(self.domOption1Input).val(self.option1Label);
		$(self.domOption2Input).val(self.option2Label);
		
		$(self.domClose).click( function() {
			self.closePanel();
			return false;	
		});
		
		$(self.domRoot).find(".ajk-verifier").each( function() {
			new AJKVerifier({
				domRootEl: this,
				submitFunc: function(data) {
					
					self.activeFieldData = data.fieldData;
					
					self.loadPage({
						fieldData: self.activeFieldData,
						page: 1
					});
					
					
					if (self.activeFieldData["opt1Search"]) {
						$(self.domSearchMessage).text(self.option1Message.replace("X_CRITERIA_X",self.activeFieldData["opt1Search"]));
					}
					else if (self.activeFieldData["opt2Search"]) {
						$(self.domSearchMessage).text(self.option2Message.replace("X_CRITERIA_X",self.activeFieldData["opt2Search"]));
					}

				}
			}).init();

		});
		
		$(self.domNextPage).click( function() {
			if (!self.currentlySubmitting) {
				self.loadPage({
					fieldData: self.activeFieldData,
					page: self.currentPage+1
				});
			}
			return false;	
		});
		
		$(self.domPrevPage).click( function() {
			if (!self.currentlySubmitting) {
				self.loadPage({
					fieldData: self.activeFieldData,
					page: self.currentPage-1
				});
			}
			return false;	
		});
		
		$(self.domFade).click( function() {
			self.closePanel();
		});
		return self;	
	},
	loadPage: function(data) {
		var self = this;
		if (self.currentlySubmitting) {
			return;
		}
		self.currentlySubmitting = true;
		self.showFader({ instantly: true });
		
		var newPage = data.page;
		var fieldData = data.fieldData;
		self.currentPage = newPage; 
		self.processFeedFunc({
			fieldData: fieldData,
			page: newPage,
			callback: function(data) {
				self.displayItems({ items: data.itemObjs });
				setTimeout( function() {
					self.currentlySubmitting = false;
					self.hideFader();
				},500);
			}
		});
	},
	setNumberOfPages: function(data) {
		var self = this;
		self.numPages = (data.value > 1000) ? 1000 : data.value;
		self.updateControls();
	},
	updateControls: function() {
		var self = this;
		$(self.domNumPages).text(self.numPages);	
		$(self.domCurrentPage).text(self.currentPage);
		if (self.currentPage == 1) {
			$(self.domPrevPage).css({ visibility: "hidden" });	
		}
		else {
			$(self.domPrevPage).css({ visibility: "visible" });	
		}
		if (self.currentPage >= self.numPages) {
			$(self.domNextPage).css({ visibility: "hidden" });	
		}
		else {
			$(self.domNextPage).css({ visibility: "visible" });	
		}
		$(self.domControls).css({ visibility: "visible" });
	},
	displayItems: function(data) {
		var self = this;
		var items = data.items;
		$(self.domItemCarouselStage).empty();
		
		if (items) {
			$.each(items, function() {
				$(self.domItemCarouselStage).append(this.domEl);
				var thisItem = this;
				$(this.domEl).find(".ajk-fis-select-click").click( function() {
					self.itemSelectedFunc({ item: thisItem });
					return false;
				});
			});
		}
		else {
			$(self.domControls).css({ visibility: "hidden" });
			$(self.domPrevPage).css({ visibility: "hidden" });	
			$(self.domNextPage).css({ visibility: "hidden" });	
		}
	},
	showFader: function(data) {
		var self = this;
		var animTime = (data && data.instantly) ? 0 : 500;
		$(self.domItemCarouselFade).css({opacity: 0, display: "block" }).animate({ opacity: 1},animTime);	
	},
	hideFader: function() {
		var self = this;
		$(self.domItemCarouselFade).animate({ opacity: 0},500, function() {
			$(this).css({opacity: 0, display: "none" })	
		});	
	},
	openPanel: function() {
		var self = this;
		theAJKWindowResizeEvent.registerAsObserver({ observer: self });
		$(self.domRoot).css({ visibility: "hidden", display: "block" });
		var viewportSize = AJKHelpers.viewportSize();
		self.sizeComponents({ viewportSize: viewportSize });
		$(self.domRoot).css({ visibility: "visible" });
		if ($.browser.msie) {
			$(self.domOption1Input).focus();
			$(self.domOption1Input).blur(); 
			$(self.domOption2Input).focus(); 
			$(self.domOption2Input).blur(); 
		}
	},
	closePanel: function() {
		var self = this; 
		$(self.domRoot).css({ display: "none" });	
		theAJKWindowResizeEvent.removeAsObserver({ observer: self });
	},
	windowDidResize: function(data) {
		var self = this;
		var newSize = data.newSize;
		self.sizeComponents({ viewportSize: newSize });
	},
	sizeComponents: function(data) {
		var self = this;
		var viewportSize = data.viewportSize;
		$(self.domRoot).css({ width: viewportSize.width, height: viewportSize.height });
		self.panelHeight = (self.panelHeight) ? self.panelHeight : $(self.domPanel).height();
		$(self.domPanel).css({ top: parseInt(viewportSize.height-self.panelHeight)/2 });
	}
}