/* Copyright 2013 Webalon Ltd
 *
 * It is strictly forbidden to copy, distribute, modify or reuse this file or its
 * contents without written permission from the copyright holder, Webalon Ltd
 *
 **************************************************************************************/


var AJKImageGallery = function(data) {
	var self = this;
	self.domRootEl = data.domRootEl;
	
	self.imageInfoFunction = data.imageInfoFunction;
	self.domImageStorage = $(self.domRootEl).find(".ak-gallery-image-storage").get()[0];
	
	self.domImages = "";
	self.numImages = "";
	self.createImageInfoHTML = data.createImageInfoHTML;
	self.imageWasSelected = data.imageWasSelected;
	self.createDomImage = data.createDomImage;
	self.buttonHideClass = data.buttonHideClass;
	
	self.domNextButton = data.domNextButton;
	self.domPrevButton = data.domPrevButton;
	self.images = new Array();
	self.gallerySize = {
		width: $(this.domRootEl).width(),
		height: $(this.domRootEl).height()	
	}
	self.domStageHolder = data.domStageHolder;
	self.domStage = data.domStage;
	self.selectedImageIndex = 0;
	self.previousImage = false;
	self.transition = AJKImageTransitions.makeTransition({ transition: (data.transition || "SimpleSwitch") }); // default
	self.currentlyAnimating = false;
	self.imageHolderHTML = data.imageHolderHTML;

	self.thumbBlockSize = data.thumbBlockSize;
	self.domThumbsHolder = data.domThumbsHolder;
	self.domThumbsStage = data.domThumbsStage;
	self.createThumbBlockFunc = data.createThumbBlockFunc;
	self.galleryThumbsController = "";
	self.selectedThumbClass = data.selectedThumbClass;
	self.domThumbNextButton = data.domThumbNextButton;
	self.domThumbPrevButton = data.domThumbPrevButton;
	self.resizeCallbackFunc = data.resizeCallbackFunc;
	self.thumbsShowing = true;
	self.animatingClass = "tl-gallery-animating";
	return self;
}

AJKImageGallery.prototype = {
	init: function() {
		var self = this;
		
		$(self.domStageHolder).css( {
			width: self.gallerySize.width+"px",
			height:	self.gallerySize.height+"px"
		});
		$(self.domStage).css( {
			width: self.gallerySize.width+"px",
			height:	self.gallerySize.height+"px"
		});
		
		$(self.domNextButton).click( function() {
			self.displayNextImage();
			return false;
		});
		$(self.domPrevButton).click( function() {
			self.displayPrevImage();
			return false;
		});
		
		if (self.domThumbsHolder) {
			self.galleryThumbsController = new AJKGalleryThumbsController({
				domRoot: self.domThumbsHolder,
				domStage: self.domThumbsStage,
				thumbBlockSize: self.thumbBlockSize,
				createThumbBlockFunc: self.createThumbBlockFunc,
				controller: self,
				selectedThumbClass: self.selectedThumbClass,
				domThumbNextButton: self.domThumbNextButton,
				domThumbPrevButton: self.domThumbPrevButton
			}).init();		
		}

		// Initialise transition object
		
		self.transition.init( {
			domStage: self.domStage,
			displaySize: self.gallerySize
		});

		return self;
	},
	reset: function() {
		var self = this;
		$(self.domStage).empty();
		self.images = new Array();
		self.domImages = new Array();	
		self.numImages = 0;		
		self.selectedImageIndex = 0;
		self.previousImage = false;
		self.currentlyAnimating = false;
	},
	loadWithImages: function(data) {
		var self = this;
		var clearTransition = data.clearTransition;
		var originalImages = data.images;
		
		if (clearTransition) {
			self.transition.previousImage = false;
		}

		self.reset();
		
		var counter = 0;		

		self.numImages = originalImages.length;
		
		if (self.numImages < 2) {
			$(self.domNextButton).addClass(self.buttonHideClass);
			$(self.domPrevButton).addClass(self.buttonHideClass);
		}
		else {
			$(self.domNextButton).removeClass(self.buttonHideClass);
			$(self.domPrevButton).removeClass(self.buttonHideClass);
		}

		$.each(originalImages, function() {
			var imageInfoHTML = self.createImageInfoHTML({ image: this });
			var aDomImage = self.createDomImage({ image: this });
			var imageOffset = this.thumbPosition;
			self.domImages.push(aDomImage);
			var anImage = new AJKImage({
				gallery: self,
				displaySize: self.gallerySize,
				domImage: aDomImage,
				imageInfoFunction: self.imageInfoFunction,
				imageInfoHTML: imageInfoHTML,
				imageHolderHTML: self.imageHolderHTML,
				index: counter++,
				imageOffset: imageOffset
			}).init();
			
			$(aDomImage).addClass("image-item");
			$(self.domImageStorage).prepend(anImage.html);
			self.images.push(anImage);
		});

		self.galleryThumbsController.loadWithImages({
			images: self.images
		});

		self.displayImageForIndex( { index: 0 });

	},
	hideThumbs: function() {
		var self = this;
		if (self.thumbsShowing) {
			self.thumbsShowing = false;
			$(self.domThumbsHolder).addClass("tl-g-thumb-holder-invisible");	
		}
	},
	showThumbs: function() {
		var self = this;	
		if (!self.thumbsShowing) {
			self.thumbsShowing = true;
			$(self.domThumbsHolder).removeClass("tl-g-thumb-holder-invisible");	
		}
	},
	displayImageForIndex: function(data) {
		var self = this;
		var index = data.index;
		var direction = data.direction;
		self.selectedImageIndex = index;
		if (self.galleryThumbsController) {
			self.galleryThumbsController.imageAtIndexWasClicked({ imageIndex: index });
			
		};
		self.currentlyAnimating = true;	
		$(self.domRootEl).addClass(self.animatingClass);
		self.transition.showImage({
			anImage: self.images[index],
			direction: direction,
			callback: function() {
				$(self.domRootEl).removeClass(self.animatingClass);
				self.currentlyAnimating = false;	
			}
		});
		self.previousImage = self.images[index];
		
		if (self.imageWasSelected) {
			self.imageWasSelected({ anImage: self.previousImage });	
		}
	},
	displayNextImage: function() {
		var self= this;
		if (self.currentlyAnimating) {
			return false;	
		}
		newIndex = (self.selectedImageIndex >= (self.numImages-1)) ? 0 : self.selectedImageIndex + 1;
		self.displayImageForIndex( { index: newIndex, direction: "forward" });
	},
	displayPrevImage: function() {
		var self= this;
		if (self.currentlyAnimating) {
			return false;	
		}
		newIndex = (self.selectedImageIndex >= 1) ? self.selectedImageIndex - 1 : (self.numImages-1);
		self.displayImageForIndex( { index: newIndex, direction: "backward" });
	},
	updateSizeTo: function(data) {
		var self = this;
		var size = data.size;
		self.gallerySize = size;
		$(self.domRootEl).css({height: size.adjustmentHeight});
		$(self.domStageHolder).css({height: size.height, width: size.width});
		$.each( self.images, function() {
			this.resizeTo({
				size: size
			});	
		});
		self.transition.updateDisplaySize({
			size: size
		});
		if (self.resizeCallbackFunc) {
			self.resizeCallbackFunc({ newSize: size });	
		}
	}
}

var AJKGalleryThumbsController = function(data) {
	var self = this;	
	self.domRoot = data.domRoot;
	self.domStage = data.domStage;
	self.thumbBlockSize = data.thumbBlockSize;
	self.createThumbBlockFunc = data.createThumbBlockFunc;
	self.images = "";
	self.controller = data.controller;
	self.viewportWidth = $(self.domRoot).width();
	self.selectedThumb = 0;
	self.domThumbs = new Array();
	self.thumbClass = "ajk-gallery-thumb-block";
	self.selectedThumbClass = data.selectedThumbClass;
	self.domThumbNextButton = data.domThumbNextButton;
	self.domThumbPrevButton = data.domThumbPrevButton;
	self.stageWidth = 0;
	self.numItems = 0;
	self.stageWidth = 0;
	self.numVisibleItems = 0;
	self.leftMostItem = 0;
	self.currentlyAnimating = false;
}

AJKGalleryThumbsController.prototype = {
	init: function() {
		var self = this;
		
		$(self.domStage).click( function(e) {
			
			var thumbBlock = AJKHelpers.getSelfOrFirstParantOfClass({
				domEl: e.target,
				className: self.thumbClass
			});
			
			if (thumbBlock) {
				var thumbIndex = parseInt($(thumbBlock).attr("thumbIndex"));
				self.thumbAtIndexWasClicked({
					thumbIndex: thumbIndex
				});
			}
		});
		
		
		$(self.domThumbNextButton).click( function() {
			self.scrollToLeftMostItemIndex({
				itemIndex: (self.leftMostItem + self.numVisibleItems)
			});
			return false;
		});

		$(self.domThumbPrevButton).click( function() {
			self.scrollToLeftMostItemIndex({
				itemIndex: (self.leftMostItem - self.numVisibleItems)
			});
			return false;
		});

		return self;
	},
	reset: function() {
		var self = this;	
		$(self.domStage).empty();
		self.selectedThumb = 0;
		self.domThumbs = new Array();
		self.numItems = 0;
		self.leftMostItem = 0;
		$(self.domStage).css({ left: 0 });
	},
	loadWithImages: function(data) {
		var self = this;
		self.images = data.images;
		
		self.reset();

		var thumbCounter = 0;

		self.updateCarouselValues();
		self.updateControls();
		
		$(self.domStage).css({ width: self.images.length * (self.thumbBlockSize.width+5) });
		
		$.each(self.images, function() {
			var thisImage = this;
			var domThumbImage = $(this.domImage).clone().get()[0];
			var aDomThumb = self.createThumbBlockFunc({
				domImage: domThumbImage	
			});
			
			$(aDomThumb).attr("thumbIndex",thumbCounter);
			$(aDomThumb).addClass(self.thumbClass);
			self.domThumbs.push(aDomThumb);
						
			$(self.domStage).append(aDomThumb);

			new AJKImageLoader({
				imageUrl: $(domThumbImage).attr("src"),
				loadCallback: function() {
					// We now need to resize the image to fit perfecty in the thumb block
					AJKHelpers.sizeImageToFitInBoxOfSize({
						domImage: domThumbImage,
						boxSize: self.thumbBlockSize,
						imageOffset: thisImage.imageOffset
					});
					//$(domThumbImage).animate({ opacity: 1},500);			
					$(domThumbImage).css({ opacity: 1});			
					
				} 
			}).init();
			
			thumbCounter++;

		});
		
		$(self.domThumbs[self.selectedThumb]).addClass(self.selectedThumbClass);

		
	},
	updateCarouselValues: function() {
		var self = this;
		self.numItems = self.images.length;
		self.stageWidth = (self.thumbBlockSize.width+self.thumbBlockSize.widthPadding) * self.numItems;
		self.numVisibleItems = parseInt(self.viewportWidth/self.thumbBlockSize.width);
	},
	scrollToLeftMostItemIndex: function(data) {
		var self = this;
		if (self.currentlyAnimating) {
			return;	
		}
		var prevLeftMostItem = self.leftMostItem;
		self.currentlyAnimating = true;
		var itemIndex = data.itemIndex;
		itemIndex = (itemIndex >= (self.numItems-self.numVisibleItems)) ? self.numItems-self.numVisibleItems : itemIndex;
		itemIndex = (itemIndex < 0) ? 0 : itemIndex;
		self.leftMostItem = itemIndex;
		var distance = Math.abs(self.leftMostItem-prevLeftMostItem);
		var animSpeed = distance * 200;
		$(self.domStage).animate({ left: - (self.thumbBlockSize.width+self.thumbBlockSize.widthPadding) * self.leftMostItem },animSpeed, function() {
			self.currentlyAnimating = false;
		});
		self.updateControls();
	},
	updateControls: function() {
		var self = this;	
		if (self.leftMostItem == 0) {
			$(self.domThumbPrevButton).css({ display: "none" });
		}
		else {
			$(self.domThumbPrevButton).css({ display: "block" });
		}
		if (self.leftMostItem + self.numVisibleItems < self.numItems) {
			$(self.domThumbNextButton).css({ display: "block" });
		}
		else {
			$(self.domThumbNextButton).css({ display: "none" });
		}
	},
	imageAtIndexWasClicked: function(data) {
		var self = this;
		var imageIndex = data.imageIndex;	
		$(self.domThumbs[self.selectedThumb]).removeClass(self.selectedThumbClass);
		$(self.domThumbs[imageIndex]).addClass(self.selectedThumbClass);
		self.selectedThumb = imageIndex;
	},
	thumbAtIndexWasClicked: function(data) {
		var self = this;
		var thumbIndex = data.thumbIndex;
		

		if (thumbIndex == self.selectedThumb || self.controller.currentlyAnimating) {
			return;	
		}
		var direction = (thumbIndex > self.selectedThumb) ? "forward" : "backward";
		
		self.controller.displayImageForIndex({ 
			index: thumbIndex,
			direction: direction
		});
		$(self.domThumbs[self.selectedThumb]).removeClass(self.selectedThumbClass);
		$(self.domThumbs[thumbIndex]).addClass(self.selectedThumbClass);
		self.selectedThumb = thumbIndex;
	}
}


var AJKImage = function(data) {
	this.gallery = data.gallery;
	this.displaySize = data.displaySize;
	this.domImage = data.domImage;
	this.imageInfoHTML = data.imageInfoHTML;
	this.imageInfoFunction = data.imageInfoFunction;
	this.domPhotoCredit="";
	this.index = data.index;
	this.imageHolderHTML = data.imageHolderHTML; 
	this.imageOffset = data.imageOffset;
	this.size = {
		width: 0,
		height: 0	
	}
	this.adjustedDimensions = {
		width: 0,
		height: 0,
		xOffset: 0,
		yOffset: 0
	}
	this.domHolder = $(this.imageHolderHTML).get()[0];
	this.html = this.domHolder; 
	this.loaded = false;
	return this;	
}

AJKImage.prototype = {
	init: function() {
		var self = this;
		$(this.domHolder).css({
			width: this.displaySize.width, 
			height: this.displaySize.height
		});

		$(self.domHolder).append(self.domImage);

		new AJKImageLoader({
			imageUrl: $(self.domImage).attr("src"),
			loadCallback: function() {
				setTimeout(function() { // Timeout needed to get size
					// We now need to resize the image to fit perfecty in the thumb block
					self.size = {
						width: $(self.domImage).width(),
						height: $(self.domImage).height()
					}
					self.fitInBoxOfSize({
						boxSize: { 
							width: self.displaySize.width, 
							height: self.displaySize.height 
						}
					});
					//$(self.domImage).animate({ opacity: 1},500);			
					$(self.domImage).css({ opacity: 1});			
				});				
			} 
		}).init();
		
		/*if (self.imageInfoFunction) {
			self.imageInfoFunction({
				domAppend: self.domHolder,
				imageInfoHTML: self.imageInfoHTML
			});
		}*/
		
		$(self.domHolder).append(self.imageInfoHTML);
		return this;
	},
	fitInBoxOfSize: function(data) {
		var self = this;
		var boxSize = data.boxSize;
		AJKHelpers.sizeImageToFitInBoxOfSize({
			domImage: self.domImage,
			boxSize: boxSize,
			imageSize: {
				width: self.size.width,
				height: self.size.height	
			},
			imageOffset: self.imageOffset
		})
	},
	removeFromDom: function() {
		var self = this;
		$(self.domHolder).remove();
	},
	resizeTo: function(data) {
		var self = this;
		self.displaySize = data.size;
		self.fitInBoxOfSize({ 
			boxSize: self.displaySize
		});
		$(self.html).css({width:self.displaySize.width, height:self.displaySize.height });
		
	}
}