/* Copyright 2013 Webalon Ltd
 *
 * It is strictly forbidden to copy, distribute, modify or reuse this file or its
 * contents without written permission from the copyright holder, Webalon Ltd
 *
 **************************************************************************************/

var AJKValueSlider = function(data) {
	var self = this;
	self.domRoot = data.domRoot;
	self.domDragger = data.domDragger;
	self.limitFunc = data.limitFunc;
	self.dragger = "";
	self.value = "";
	self.range = data.range;
	self.callback = data.callback;
	self.name = data.name;
}

AJKValueSlider.slidersByName = [];

AJKValueSlider.prototype = {
	init: function() {
		var self = this;
		AJKValueSlider.slidersByName[self.name] = self;
		
		self.dragger = new AJKDraggable({
			domDragEl: self.domDragger,
			limitFunc: self.limitFunc,
			mouseMoveFunc: function(data) {
				var newVal = data.dragElPos.x/self.limitFunc().maxX * (self.range.max - self.range.min) + self.range.min;
				self.callback({ newVal: newVal });
			},
			dragDidStartFunc: function() {

			},
			dragDidEndFunc: function(data) {
			}
		}).init();
				
		return self;
	},
	setValue: function(data) {
		var self = this;
		self.value = (data.value) ? parseFloat(data.value) : self.range.min;
		var maxPos = self.limitFunc().maxX;
		var pos = (self.value-self.range.min)/(self.range.max-self.range.min) * maxPos;
		$(self.domDragger).css({ left: pos });
	}
}

var AJKVerifier = function(data) {
	this.domRootEl = data.domRootEl;
	this.domFields = $(this.domRootEl).find(".ajk-verifier-field").get();
	this.domSubmitButton = $(this.domRootEl).find(".ajk-verifier-submit").get()[0];
	this.domCancelButton = $(this.domRootEl).find(".ajk-verifier-cancel").get()[0];
	this.domRevertButton = $(this.domRootEl).find(".ajk-verifier-revert").get()[0];
	this.domTooltipDisplay = $(this.domRootEl).find(".ajk-verifier-tooltip-displayer").get()[0];
	this.domErrorMessage = $(this.domRootEl).find(".ajk-verifier-error-message").get()[0];
	this.domSuccessMessage = $(this.domRootEl).find(".ajk-verifier-success-message").get()[0];
	this.defaultErrorMessage = $(this.domErrorMessage).text(); 
	this.submitFunc = data.submitFunc;
	this.cancelFunc = data.cancelFunc;
	this.revertFunc = data.revertFunc;
	this.objFields = new Array();
	this.objFieldsByKey = new Array();
	this.numFields = this.domFields.length;
	this.onChangeCallback = "";
	this.onKeyUpCallback = "";
	this.savedFieldValues = "";
	this.disableSaveAndRevertClass = "ajk-verifier-disable-save-and-revert";
	self.saveAndRevertDisabled = false;
	return this;
}

// When a successful save takes place, we need to overwrite the saved values with the current values
//     - Admin Controller needs to a call back function we call with success = true or false
// At that point we also need to disable the revert and save buttons
// When a change occurs we activate the save and revert buttons

AJKVerifier.prototype = {
	init: function() {
		var self = this;	
		
		$(this.domRootEl).find("form").each( function() {
			$(this).submit( function() { return false } );	
		});
		
		var counter = 0;
		$.each(self.domFields, function() {
			var isLast = (++counter == self.numFields) ? true : false;
			var aNewField = new AJKVerifierField({
				domRootEl: this,
				isLast: isLast,
				fieldHoverFunc: function(data) {
					self.displayTooltip(data);
				},
				clearAllErrorsFunc: function(data) {
					self.clearAllErrors(data);	
				},
				controller: self
			}).init();
			
			self.objFields.push(aNewField);
			self.objFieldsByKey[aNewField.fieldName] = aNewField; 
			
		});
		
		if (!self.domErrorMessage) {
			self.hideErrorMessage = function() {};
			self.displayErrorMessage = function() {};
		}

		if (!self.domTooltipDisplay) {
			self.displayTooltip	= function() {};
		}

		$(self.domSubmitButton).click( function() {
			self.attemptToSubmit();
			return false;
		});
		
		$(self.domCancelButton).click( function() {
			if (self.cancelFunc) {
				self.cancelFunc();	
			}
			return false;
		});

		$(self.domRevertButton).click( function() {
			if (self.revertFunc && !self.saveAndRevertDisabled) {
				self.revertFunc();
				self.clearAllErrors();	
			}
			return false;
		});

		return self;
	},
	fireButton: function(data) {
		var self = this;
		var buttonType = data.buttonType;
		switch (buttonType) {
			case "revert": 
				$(self.domRevertButton).click();
			break;
			case "save":
				$(self.domSubmitButton).click();
			break;
			case "cancel":
				$(self.domCancelButton).click();
			break;
		}
		
	},
	showCancelButton: function() {
		var self = this;
		if (self.domCancelButton) {
			$(self.domCancelButton).css({ display: "block" });
		}
	},
	showRevertButton: function() {
		var self = this;
		if (self.domRevertButton) {	
			$(self.domRevertButton).css({ display: "block" });
		}
	},
	hideCancelButton: function() {
		var self = this;
		if (self.domCancelButton) {
			$(self.domCancelButton).css({ display: "none" });
		}
	},
	hideRevertButton: function() {
		var self = this;
		if (self.domRevertButton) {	
			$(self.domRevertButton).css({ display: "none" });
		}
	},
	/*setStatusChanged: function() {
		var self = this;
		self.status = "changed";
		self.disableSaveAndRevert();
	},
	setStatusUnchanged: function() {
		var self = this;
		self.status = "unchanged";
		self.disableSaveAndRevert();
	},*/
	enableSaveAndRevert: function() {
		var self = this;
		$(self.domRootEl).removeClass(self.disableSaveAndRevertClass);
		self.saveAndRevertDisabled = false;
	},
	disableSaveAndRevert: function() {
		var self = this;
		$(self.domRootEl).addClass(self.disableSaveAndRevertClass);
		self.saveAndRevertDisabled = true;
	},
	setFieldValues: function(data) {
		var self = this;
		var fieldValues = data.fieldValues;
		var forceChangeEvent = data.forceChangeEvent;

	    for (var fieldName in fieldValues) {
			if (self.objFieldsByKey[fieldName]) {
				var oldValue = self.objFieldsByKey[fieldName].getValue(); 
				self.objFieldsByKey[fieldName].forceSetValue({ value: fieldValues[fieldName] });
				if (forceChangeEvent=="auto") {
					if (oldValue != fieldValues[fieldName]) {
						self.fieldDidChange({
							fieldName: fieldName,
							fieldValue: self.objFieldsByKey[fieldName].fieldValue
						});
					}
				}
				else if (forceChangeEvent) {
					self.fieldDidChange({
						fieldName: fieldName,
						fieldValue: self.objFieldsByKey[fieldName].fieldValue
					});
				}
	    	}
	    }
		
	},
	getFieldValueFromName: function(data) {
		var self = this;
		var fieldName = data.fieldName;
		return self.objFieldsByKey[fieldName].getValue();
	},
	attemptToSubmit: function() {
		var self = this;
		if (self.saveAndRevertDisabled) {
			return false;	
		}
		if (self.verifyFields()) {
			self.submit();
		}
		else {
			self.displayErrorMessage({
				message: self.defaultErrorMessage	
			})
		}
	},
	errorHighlightField: function(data) {
		var self = this;
		var fieldName = data.fieldName;
		self.objFieldsByKey[fieldName].highlightError();	
	},
	displayErrorMessage: function(data) {
		var self = this;
		var message = data.message;
		$(self.domErrorMessage).html(message).css({display:"block"});
		self.hideSuccessMessage();
	},
	hideErrorMessage: function(data) {
		var self = this;
		$(self.domErrorMessage).css({display:"none"});

	},
	displaySuccessMessage: function(data) {
		var self = this;
		if (self.domSuccessMessage) {
			$(self.domSuccessMessage).css({display:"block"});
		}
	},
	hideSuccessMessage: function(data) {
		var self = this;
		if (self.domSuccessMessage) {
			$(self.domSuccessMessage).css({display:"none"});
		}
	},
	clearAllErrors: function() {
		var self = this;
		self.hideErrorMessage();
		self.hideSuccessMessage();
		$.each(self.objFields, function() {
			this.clearError();
		});
	},
	displayTooltip: function(data) {
		var self = this;
		var tooltipText = data.tooltipText;
		if (tooltipText) {
			$(self.domTooltipDisplay).html(tooltipText).css({ display: "block" });
		}
		else {
			$(self.domTooltipDisplay).html(tooltipText).css({ display: "none" });
		}
	},
	verifyFields: function() {
		var self = this;
		var failedFields = 0;
		$.each(self.objFields, function() {
			if (!this.verify()) {
				failedFields++;
				this.highlightError();
			}
		});
		return (failedFields == 0);
	},
	ensureValidColorHex: function(data) {
		var self = this;
		var thisVal = data.colorHex;
		var regTxt = (/^(#)?([0-9a-fA-F]{3})([0-9a-fA-F]{3})?$/);
		
		if (thisVal && regTxt.test(thisVal)) {
			return thisVal.replace("#","");	
		}
		else {
			return "999999";	
		}

	},
	submit: function() {
		var self = this;
		if (self.saveAndRevertDisabled) {
			return false;	
		}

		$.each(self.objFields, function() {
			this.blur();	
		});
		
		var fieldData = new Object();
		self.displaySuccessMessage();
		$.each(self.objFields, function() {
			var thisVal = this.fieldValue;
			
			if (thisVal && thisVal.replace) { // is a string
				if (this.controlType == "input text") {
					thisVal = thisVal.replace(/[\n\r]/g,"");
				}
				thisVal = thisVal.replace(/[\t]/g,"");
				thisVal = thisVal.replace( /\x18/g, "" );
				thisVal = thisVal.replace( /\u2028/g, "" );
				thisVal = thisVal.replace( /\u2029/g, "" );
			}
			if (this.colorPicker) {
				thisVal = self.ensureValidColorHex({ colorHex: thisVal }); 	
			}
			fieldData[this.fieldName] = thisVal;
		});
		self.submitFunc({
			fieldData: fieldData
		});
	},
	saveFieldValues: function() {
		var self = this;
		self.savedFieldValues = {};
		$.each(self.objFields, function() {
			self.savedFieldValues[this.fieldName] = this.fieldValue;
		});
	},
	clearSavedFieldValues: function() {
		var self = this;
		self.savedFieldValues = {};
		$.each(self.objFields, function() {
			self.savedFieldValues[this.fieldName] = "";
		});
	},
	restoreSavedFieldValues: function() {
		var self = this;
		if (self.savedFieldValues) {
			self.setFieldValues({
				fieldValues: self.savedFieldValues,
				forceChangeEvent: "auto"
			});
		}
		self.disableSaveAndRevert();
	},
	restoreOldValueForField: function(data) {
		var self = this;
		var fieldName = data.fieldName;
		self.objFieldsByKey[fieldName].forceSetValue({ value: self.objFieldsByKey[fieldName].oldFieldValue });
		if (!self.areValuesDifferentFromSavedValues()) {
			self.disableSaveAndRevert();
		}
	},
	restoreSavedValueForField: function(data) {
		var self = this;
		var fieldName = data.fieldName;
		if (self.savedFieldValues && self.savedFieldValues[fieldName]) {
			self.objFieldsByKey[fieldName].forceSetValue({ value: self.savedFieldValues[fieldName] });
			self.fieldDidChange({
				fieldName: fieldName,
				fieldValue: self.objFieldsByKey[fieldName].fieldValue
			});

			if (!self.areValuesDifferentFromSavedValues()) {
				self.disableSaveAndRevert();
			}
		}
	},
	clearFields: function() {
		var self = this;
		$.each(self.objFields, function() {
			this.clear();
		});
	},
	fieldDidChange: function(data) {
		var self = this;
		var fieldName = data.fieldName;
		var fieldValue = data.fieldValue;

		if (self.onChangeCallback) {
			self.onChangeCallback({
				fieldName: fieldName,
				fieldValue: fieldValue
			});
		}

		if (self.areValuesDifferentFromSavedValues()) {
			self.enableSaveAndRevert();
		}
		else {
			self.disableSaveAndRevert();
		}
		
	},
	fieldDidKeyUp: function(data) {
		var self = this;
		var fieldName = data.fieldName;
		var fieldValue = data.fieldValue;

		if (self.onKeyUpCallback) {
			self.onKeyUpCallback({
				fieldName: fieldName,
				fieldValue: fieldValue
			});
		}
				
		if (self.areValuesDifferentFromSavedValues()) {
			self.enableSaveAndRevert();
		}
		else {
			self.disableSaveAndRevert();
		}
		
	},
	isValueDifferentFromSavedValueForField: function(data) {
		var self = this;
		var fieldName = data.fieldName;
		if (self.savedFieldValues) {
            var val = self.objFieldsByKey[fieldName].getValue();
            var savedVal = self.savedFieldValues[fieldName];
			return (val != savedVal && !(!val && !savedVal));
		}
		else {
			return true;	
		} 	
	},
	areValuesDifferentFromSavedValues: function() {
		var self = this;
		if (self.savedFieldValues) {
			var difference = false;
			$.each(self.objFields, function() {
                var val = this.getValue();
                var savedVal = self.savedFieldValues[this.fieldName];
				if (val != savedVal && !(!val && !savedVal)) {
					difference = true;
				}
			});
			return difference;
		}
		else {
			// No values stored so assume values are different;
			return true;	
		}
	}
}

var AJKVerifierField = function(data) {
	this.domRootEl = data.domRootEl;
	this.fieldHoverFunc = data.fieldHoverFunc;
	this.clearAllErrorsFunc = data.clearAllErrorsFunc;
	this.isLast = data.isLast;
	this.controller = data.controller;
	this.domFormat = "";
	this.domChars = "";
	this.domTooltip = "";
	this.domControl = "";
	this.tooltip = "";
	this.format = "";
	this.chars = "";
	this.defaultValue = "";
	this.controlType = "";
	this.fieldValue = "";
	this.fieldName = "";
	this.defaultValueOkay = "";
	this.valueOptions = "";
	this.colorPicker = false;
	this.oldFieldValue = "";
	return this;
}

AJKVerifierField.prototype = {
	init: function() {
		var self = this;
		self.domFormat = $(this.domRootEl).find(".ajk-verifier-format");
		self.domChars = $(this.domRootEl).find(".ajk-verifier-chars");
		self.domTooltip = $(this.domRootEl).find(".ajk-verifier-tooltip");
		self.domControl = $(this.domRootEl).find(".ajk-verifier-control");
		self.tooltip = (self.domTooltip.length > 0) ? $(self.domTooltip).text() : "";
		self.format = (self.domFormat.length > 0) ? $(self.domFormat).text() : "";
		self.chars = (self.domChars.length > 0) ? $(self.domChars).text() : "";
		self.domControl = (self.domControl.length > 0) ? $(self.domControl).get()[0] : "";
		self.domOverlay = $(self.domRootEl).find(".ajk-verifier-overlay").get()[0];
		self.defaultValueOkay = ($(this.domRootEl).find(".ajk-verifier-not-default-value").length > 0) ? false : true;
		self.hideDefaultValue = ($(this.domRootEl).find(".ajk-verifier-hide-default-value").length > 0) ? true : false;
		
		var domOptionValues = $(this.domRootEl).find(".ajk-verifier-value-options");
		self.valueOptions = (domOptionValues.length > 0) ? $(domOptionValues).text().split() : "";
		
		self.fieldName = $(self.domControl).attr("name");
				
		if (self.chars) {
			var charArray = self.chars.split("-");
			self.chars = {
				minChar: charArray[0],
				maxChar: charArray[1]	
			}
		}
		self.calculateControlType();
		self.setupValueGetterSetter();
		self.setupVerificationFunction();
		self.setupTooltip();
		self.setupFocus();
		self.setupClick();
		self.setupKeyEvents();
		self.invalidDateMessage = (TLConfigText && TLConfigText['verifier_invaldiDate_message']) ? TLConfigText['verifier_invaldiDate_message'] : "Not a valid date";
		
		
		if ($(self.domControl).hasClass("ajk-color-selector") && typeof AJKColorPicker != "undefined" ) {
			self.colorPicker = new AJKColorPicker.color(self.domControl,{});
		}

		if (self.domOverlay) {
			self.initOverlay();	
		}
		return self;
	},
	initOverlay: function() {
		var self = this;
		$(self.domOverlay).click( function() {
			$(this).css({ display: "none" });
			$(self.domControl).focus();
		});
		
		$(self.domControl).focus( function() {
			$(self.domOverlay).css({ display: "none" });
		});
		
		$(self.domControl).blur( function() {
			if ($(this).val() == "") {
				$(self.domOverlay).css({ display: "block" });
			}
		});
	},
	setupValueGetterSetter: function() {
		var self = this;
		switch (self.controlType) {
			case "span ajk-custom-checkbox":
				self.getValue = function() {
					return $(this.domControl).text();	
				}
				self.setValue = function(data) {
					$(this.domControl).text(data.value);	
				}
			break;
			case "span ajk-hidden-value":
				self.getValue = function() {
					return $.data(this.domControl, "value");
				}
				self.setValue = function(data) {
					$.data(this.domControl, "value",data.value);
				}
			break;
			case "textarea textarea":
				self.getValue = function() {
					return $(this.domControl).val();	
				}
				self.setValue = function(data) {
					$(this.domControl).val(data.value);	
				}
			break;
			case "select":
				self.getValue = function() {
					return $(this.domControl).val();	
				}
				self.setValue = function(data) {
					$(this.domControl).val(data.value);	
				}
			break;
			default:
				self.getValue = function() {
					return $(this.domControl).val();	
				}
				self.setValue = function(data) {
					$(this.domControl).val(data.value);	
					self.updateValueSlider({ value: data.value });
				}
			break;
		}
	},
	updateValueSlider: function(data) {
		var self = this;
		var thisSlider = AJKValueSlider.slidersByName[self.fieldName]; 
		if (thisSlider) {
			thisSlider.setValue({ value: data.value });
		}	
	},
	calculateControlType: function() {
		var self = this;
		var tagName = self.domControl.tagName;
		var type = $(self.domControl).attr("type");
		self.controlType = (tagName+" "+type).toLowerCase();
		if (tagName == "SELECT") {
			self.controlType = "select";	
		}
		if (self.controlType == "input password") {
			self.controlType = "input text";
		}
	},
	setupClick: function() {
		var self = this;
		switch (self.controlType) {
			case "span ajk-custom-checkbox":
			self.fieldValue = self.getValue();
			$(self.domControl).click( function() {
				if (self.fieldValue == "on") {
					self.fieldValue = "off";
					$(this).removeClass("checkbox-selected");	
				}
				else {
					self.fieldValue = "on";
					$(this).addClass("checkbox-selected");	
				}
				self.clearAllErrorsFunc();
				return false;
			});
			break;
			default:
			break;
		}		
	},
	setupFocus: function() {
		var self = this;
		switch (self.controlType) {
			case "input text":
				self.fieldValue = self.defaultValue = $(self.domControl).val();
				$(self.domControl).focus( function() {
					var thisVal = self.getValue();
					if (self.format == "date" && thisVal == self.invalidDateMessage) {
						self.setValue({value:""});	
					}
					
					if (thisVal == self.defaultValue && (!self.defaultValueOkay || self.hideDefaultValue) ) {
						self.setValue({value:""});	
					}
					$(self.domRootEl).addClass("ajk-verifier-field-focused");
					self.clearAllErrorsFunc();
					
				}).blur( function() {
					var thisVal = self.getValue();
					if (self.format == "date") {
						var parsedDate = Date.parse(thisVal);
						if (parsedDate) {
							var formatedDate = AJKHelpers.formatDate({
								formatString: "DD MMM YYYY HH:mm:ss",
								date: parsedDate,
								language: "base"
							});
							self.setValue({value: formatedDate });
							thisVal = formatedDate;
						}
						else {
							self.setValue({value: self.invalidDateMessage });	
							thisVal = self.invalidDateMessage;
						}						
					}

					if (!thisVal) {
						self.setValue({value:self.defaultValue});	
					}
					self.oldFieldValue = self.fieldValue; 
					self.fieldValue = thisVal;
					
					if (self.oldFieldValue != thisVal) {
						self.controller.fieldDidChange({
							fieldName: self.fieldName,
							fieldValue: thisVal
						});
					}
					
					$(self.domRootEl).removeClass("ajk-verifier-field-focused");
					
				});
			break;
			case "textarea textarea":
				self.fieldValue = self.defaultValue = $(self.domControl).val();
				$(self.domControl).focus( function() {
					var thisVal = self.getValue();
					if (thisVal == self.defaultValue) {
						//self.setValue({value:""});	
					}
					$(self.domRootEl).addClass("ajk-verifier-field-focused");
					self.clearAllErrorsFunc();
					
				}).blur( function() {
					var thisVal = self.getValue();
					if (!thisVal) {
						//self.setValue({value:self.defaultValue});	
					}

					if (self.fieldValue != thisVal) {
						self.controller.fieldDidChange({
							fieldName: self.fieldName,
							fieldValue: thisVal
						});
					}

					self.fieldValue = thisVal;
					$(self.domRootEl).removeClass("ajk-verifier-field-focused");
				});
			break;
			case "select":
				self.fieldValue = $(self.domControl).val(); 
				$(self.domControl).focus( function() {
					$(self.domRootEl).addClass("ajk-verifier-field-focused");
					self.clearAllErrorsFunc();
					
				}).blur( function() {
					var thisVal = self.getValue();
					if (self.fieldValue != thisVal) {
						self.controller.fieldDidChange({
							fieldName: self.fieldName,
							fieldValue: thisVal
						});
					}
					self.fieldValue = thisVal;
					$(self.domRootEl).removeClass("ajk-verifier-field-focused");
				}).change( function() {
					$(self.domControl).blur();
				});
			break;
			case "span ajk-hidden-value":
				$(self.domControl).blur( function() {
					var thisVal = self.getValue();
					if (self.fieldValue != thisVal) {
						self.controller.fieldDidChange({
							fieldName: self.fieldName,
							fieldValue: thisVal
						});
					}
					self.fieldValue = thisVal;
				}).change( function() {
					$(self.domControl).blur();
				});
			default:
			break;
		}
	},
	setupKeyEvents: function() {
		var self = this;
		
		if (self.format == "number") {
			$(self.domControl).keydown( function(e) {
				var charCode = e.keyCode;
				if ((charCode >= 48 && charCode <=57) || charCode == 8 || charCode == 13 || charCode == 9 || charCode == 37 || charCode == 39 || charCode == 46 || charCode == 17 || charCode == 18) {
					return true;
				}
				else {
					return false;	
				}
			});
		}
		
		$(self.domControl).keyup( function(e) {
			self.controller.fieldDidKeyUp({
				fieldName: self.fieldName,
				fieldValue: $(this).val()
			});
		});
		
		if (self.isLast && self.controlType == "input text") {
			$(self.domControl).keyup( function(e) {
				if (e.keyCode == 13) {
					self.fieldValue = $(self.domControl).val();
					self.controller.attemptToSubmit();
				}
			});
		}	
	},
	clear: function() {
		var self = this;
		switch (self.controlType) {
			case "input text":
				self.fieldValue = (self.defaultValue) ? self.defaultValue : "";
				$(self.domControl).val(self.fieldValue);
				if (self.domOverlay) {
					$(self.domOverlay).css({ display: "block" });	
				}
			break;
			case "textarea textarea":
				self.fieldValue = (self.defaultValue) ? self.defaultValue : "";
				$(self.domControl).val(self.fieldValue);
			break;
			default:
				self.fieldValue = (self.defaultValue) ? self.defaultValue : "";
				$(self.domControl).val(self.fieldValue);
			break;
		}
	},
	forceSetValue: function(data) {
		var self = this;
		var value = data.value;
		switch (self.controlType) {
			case "input text":
				self.oldFieldValue = self.fieldValue; 
				self.fieldValue = value;
				$(self.domControl).val(value);
				self.updateValueSlider({ value: value });
			break;
			case "textarea textarea":
				self.oldFieldValue = self.fieldValue; 
				self.fieldValue = value;
				$(self.domControl).val(value);
			break;
			case "span ajk-hidden-value":
				self.oldFieldValue = self.fieldValue; 
				self.fieldValue = value;
				$.data(self.domControl,"value",value);
			break;
			default:
				self.oldFieldValue = self.fieldValue; 
				self.fieldValue = value;
				$(self.domControl).val(value);
				self.updateValueSlider({ value: value });
			break;
		}
		
		if (self.colorPicker) {
			AJKColorPicker.fireEvent(self.domControl,"keyup");
		}
	},
	clearError: function() {
		var self = this;
		$(self.domRootEl).removeClass("ajk-verifier-field-error");
	},
	setupTooltip: function() {
		var self = this;
		$(self.domControl).hover( function() {
			self.fieldHoverFunc({
				tooltipText: self.tooltip
			});
		}, function() {
			self.fieldHoverFunc({
				tooltipText: ""
			});
		});	
	},
	setupVerificationFunction: function() {
		var self = this;
		switch (self.format) {
			case "email":
				self.verify = function() {
					var self = this; 
					var fieldValue = self.getValue();
					if (!fieldValue || (fieldValue == self.defaultValue && !self.defaultValueOkay)) {
						return false;	
					}
					return fieldValue.match(/^(("[\w-\s]+")|([\w-]+(?:\.[\w-]+)*)|("[\w-\s]+")([\w-]+(?:\.[\w-]+)*))(@((?:[\w-]+\.)*\w[\w-]{0,66})\.([a-z]{2,6}(?:\.[a-z]{2})?)$)|(@\[?((25[0-5]\.|2[0-4][0-9]\.|1[0-9]{2}\.|[0-9]{1,2}\.))((25[0-5]|2[0-4][0-9]|1[0-9]{2}|[0-9]{1,2})\.){2}(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[0-9]{1,2})\]?$)/i);
				}
			break;
			case "date": {
				self.verify = function() {
					var self = this;
					var fieldValue = self.getValue();	
					if (!fieldValue || (fieldValue == self.defaultValue && !self.defaultValueOkay)) {
						return false;	
					}
					return (Date.parse(fieldValue)) ? true : false;
				}	
			}
			break;
			default:
				self.verify = function() {
					if (!self.defaultValueOkay && self.fieldValue == self.defaultValue) {
						return false;	
					}
					return (self.checkFieldCharLength() && self.checkForValueOptions());
				}
			break;	
		}
		
	},
	verify: function() { // This will get overwritten at object creation
			
	},
	blur: function() {
		var self = this;	
		$(self.domControl).blur();
	},
	highlightError: function() {
		var self = this;
		$(self.domRootEl).addClass("ajk-verifier-field-error");
	},
	checkFieldCharLength: function() {
		var self = this;
		if (!self.chars) {
			return true;	
		}
		var fieldLength = self.fieldValue.length;
		return (fieldLength >= self.chars.minChar && fieldLength <= self.chars.maxChar);
	},
	checkForValueOptions: function() {
		var self = this;
		if (!self.valueOptions) {
			return true;	
		}
		var optionFound = false;
		$.each(self.valueOptions, function() {
			if (self.fieldValue == this) {
				optionFound = true;
			}		
		});
		return optionFound;
	}
}

