/* Copyright 2013 Webalon Ltd
 *
 * It is strictly forbidden to copy, distribute, modify or reuse this file or its
 * contents without written permission from the copyright holder, Webalon Ltd
 *
 **************************************************************************************/


// Admin augmentation for TLMainController, TLMarker

/* TLMainController
******************************************************************************************************************************************************************/
TLMainController.prototype.launchAdmin = function() {
	var self = this;	
	$("body").addClass("tl-container-logged-in");
	
	// Maybe add loaded or new timeline here
	var aTimeline = {
		urlFriendlyTitle: self.timeline.urlFriendlyTitle, 
		title: self.timeline.title,
		secretWord: self.timeline.secretWord,
		id: self.timeline.id
	}
	self.user.timelines.push(aTimeline);
	self.user.timelinesByKey[aTimeline.id] = aTimeline;

	self.initialiseAdmin();
}


TLMainController.prototype.setLanguage = function(data) {
	var self = this;
	var language = data.language;
	$(self.domMainHolder).removeClass("tl-language-"+self.timeline.language);
	self.timeline.language = language;
	$(self.domMainHolder).addClass("tl-language-"+self.timeline.language);
	self.initLanguage();
}

TLMainController.prototype.initialiseAdmin = function() {
	var self = this;	
	self.timeline.lazyLoadingActive = false; // Disable lazyloading for any new images added
	self.adminController = new TLAdminController({
		domRoot: $("#tl-admin-holder").get()[0],
		domAdminTab: $("#tl-admin-main-tab").get()[0],
		user: self.user,
		mainController: self,
		timeline: self.timeline	
	}).init();
}

TLMainController.prototype.createNewMarker = function() {
	var self = this;	
	var selectedDate = theTLSettings.currentDate;
	
	if (self.timeline.markerSpacing == "equal" && self.selectedView) {
		selectedDate = self.selectedView.getEqualSpacingDateFromStandardDate({ aDate: selectedDate });	
	}
	var startDateTime = (selectedDate.getTime() < self.timeline.startDate.getTime()) ? self.timeline.startDate : selectedDate;
	startDateTime = (startDateTime.getTime() > self.timeline.endDate.getTime()) ? self.timeline.endDate : startDateTime;
	
	var newMarker = self.createMarker({
		id: "",
		ownerId: self.user.id,
		ownerName: (self.user.secretUser) ? self.user.username : "",
		startDate: AJKHelpers.createDateWithTime({ time: startDateTime.getTime() }),
		endDate: AJKHelpers.createDateWithTime({ time: startDateTime.getTime() }),
		media: [],
		headline: TLConfigText['storyDefault_headline'],
		introText: TLConfigText['storyDefault_intro'],
		category: self.timeline.categories[0],
		dateFormat: "auto"
	});

	self.sortMarkersList();

	if (self.timeline.markerSpacing == "equal" || self.timeline.viewType == "category-band") {
		// If equal spacing, we need to recreate the entire timeline when we add a new marker
		self.updateViewsWithNewDateRangeAndZoom({ zoom: self.timeline.zoom });
	}
	else { // Default
		self.selectedView.addNewMarkerToStage({ marker: newMarker });
		self.sliderController.addPointToScale({ marker: newMarker });
	}
			
	return newMarker;
}
	
TLMainController.prototype.deleteMarker = function(data) {
	var self = this;
	var marker = data.marker;
	self.markersByKey[marker.markerKey] = "";
	self.markers = AJKHelpers.removeItemFromArray({ anArray: self.markers, item: marker  })
	// Now we need to do an ajax call to delete an item
	marker.deleteSelf();
	self.sortMarkersList();
	if (self.timeline.markerSpacing == "equal") {
		// If equal spacing, we need to recreate the entire timeline when we add a new marker
		self.updateViewsWithNewDateRangeAndZoom({ zoom: self.timeline.zoom });
	}
	else { // Default
		self.selectedView.markerWasDeleted();
	}
}




/* TLMarker 
******************************************************************************************************************************************************************/

TLMarker.prototype.isEditButtonActive = false;

TLMarker.prototype.oldFocus = TLMarker.prototype.focus;
TLMarker.prototype.focus = function(data) {
	var self = this;
	var tData = (data) ? data : "";
	self.oldFocus(tData);
	if (self.mainController.timeline.isEditable && (!self.isEditButtonActive || tData.forceUpdate) && self.mainController.adminController.adminVisible) {
		self.isEditButtonActive = true;
		self.mainController.adminController.storyEditButton.showForStory({ story: self });
	}	
}

TLMarker.prototype.oldUnfocus = TLMarker.prototype.unfocus;
TLMarker.prototype.unfocus = function(data) {
	var self = this;
	var tData = (data) ? data : "";
	self.oldUnfocus(tData);
	if (self.mainController.timeline.isEditable) {
		self.mainController.adminController.storyEditButton.remove();
	}	
	self.isEditButtonActive = false;
}



TLMarker.prototype.deleteMediaItem = function(data) {
	var self = this;	
	var mediaItem = data.mediaItem;
	
	var vars = {};
	vars["storyId"] = self.id;
	vars["mediaId"] = mediaItem.id;
	vars["timelineId"] = self.mainController.timeline.id;
	
	AJKHelpers.removeItemFromArray({ anArray: self.media, item: mediaItem });
	// MIght have to remove or update te main image
	
	self.mediaHasBeenUpdated();
}

TLMarker.prototype.createNewMediaItem = function() {
	var self = this;
	var newMediaItem = {
		id:"awaitingId",
		orderIndex: 10,
		type:"Image",
		src: TLConfigText['marker_media_Enter_meda_source_here'],
		caption: "",
		listPosition: self.media.length,
		thumbPosition: "0,0",
		mediaDataUri: ""
	}
	return newMediaItem;
}

TLMarker.prototype.mediaHasBeenUpdated = function() {
	var self = this;	
	// This is called when alterations have been made to images, videos or audio.
	//Might have to update the currently displayed image etc
	
	var oldDisplayImage = self.displayImage;
	var oldNumPhotos = self.numImages;
	self.initialiseMedia();

	$(self.domImageGallery).find(".ig-num-items-text").text(self.numImages);

	if (!self.displayImage) {
		// Image is null, so remove gallery
		$(self.domImageGallery).remove();
		self.domImageGallery = "";
		self.domMainImageHolder = "";
		$(self.domRootInner).removeClass(self.galleryClass);
		self.domMainImage = ""; 
		self.domMainImageWidth = ""; 
		self.domMainImageHeight = ""; 
	}
	else {
		$(self.domRootInner).addClass(self.galleryClass);
		
		// New image
		if (self.domMainImageHolder) {
			$(self.domMainImageHolder).empty().append('<img src="'+self.getDisplayImage().src+'" alt="" />');
			self.domMainImage = $(self.domMainImageHolder).find("img").get()[0]; 
			$(self.domMainImage).css({ visibility: "hidden" });
			new AJKImageLoader({
				imageUrl: self.getDisplayImage().src,
				loadCallback: function() {
					self.updateMainImage();
					$(self.domRoot).css({ visibility: "visible" });
					$(self.domMainImage).css({ visibility: "visible" });
				}
			}).init();
			
		}
		else {
			// There is no image gallery, so create1 one
			self.domImageGallery = $(self.generateImageGalleryHTML()).get()[0];
			self.domMainImageHolder = $(self.domImageGallery).find(".ig-inner").get()[0];
			self.domMainImage = $(self.domMainImageHolder).find("img").get()[0]; 
			// We need to create the HTML for the images
			$(self.domMarkerDate).after(self.domImageGallery);
			$(self.domMainImage).css({ visibility: "hidden" });
			new AJKImageLoader({
				imageUrl: self.getDisplayImage().src,
				loadCallback: function() {
					self.updateMainImage();
					$(self.domMainImage).css({ visibility: "visible" });
				}
			}).init();
		}
	}
		
	// We also need to update the Num Of Images in the marker image gallery
	
	// Clear 3D Image
	self.dom3DImage = "";
	self.mainController.selected3DView.redisplay();
}

TLMarker.prototype.addMediaItem = function(data) {
	var self = this;
	var mediaItem = data.mediaItem;
	self.media.push(mediaItem);
	
	self.mediaHasBeenUpdated();
	// We may want to resort the media list and refresh media at this point
}

TLMarker.prototype.setValue = function(data) {
	var self = this;	
	var valueName = data.valueName;
	var value = data.value;
	switch(valueName) {
		case "headline":
			self.headline = value;
			self.clippedHeadline = AJKHelpers.clipToMaxCharWords({ aString: self.headline, maxChars: self.HEADLINEMAXCHARS });
			if (self.domMarkerHeadline) {
				$(self.domMarkerHeadline).text(self.clippedHeadline);	
			}
			if (self.domSliderToolTipHeadline) {
				$(self.domSliderToolTipHeadline).text(self.headline);	
			}
		self.clear3DText();
		break;			
		case "introText":
			self.introText = value;
			self.clippedIntroText = AJKHelpers.clipToMaxCharWords({ aString: self.introText, maxChars: self.INTROTEXTMAXCHARS });
			self.clippedContentIntroText = (self.externalLink) ? AJKHelpers.clipToMaxCharWords({ aString: self.introText, maxChars: 300 }) : self.introText;

			if (self.domMarkerIntroText) {
				$(self.domMarkerIntroText).text(self.clippedIntroText);	
			}
		break;			
		case "startDate":
			if (value != self.startDate && value && value.getFullYear) {
				self.startDate = value; 
				self.syncStartEndDateIfNeeded();
				self.updatePositionFromDate();
				self.updateDisplayDates();
			}
		break;			
		case "endDate":
			if (value && value.getFullYear && value.getTime() != self.endDate.getTime()) {
				self.endDate = value;
				self.syncStartEndDateIfNeeded();
				self.updatePositionFromDate();
				self.updateDisplayDates();
			}
		break;			
		case "dateFormat":
			self.dateFormat = value;
			self.updateDisplayDates();
		break;			
		case "category":
			var newCat = self.mainController.timeline.categoriesByKey[self.mainController.categoriesKeyPrefix+value];
			self.category = (newCat) ? newCat : self.category;
			self.categoryHasChanged();
		break;
		default:
			self[valueName] = value;
		break;	
	}
}


TLMarker.prototype.updateDisplayDates = function() {
	var self = this;
	var dateText = self.formatDisplayDate();
	$(self.domMarkerDate).text(dateText);
	if (self.domSliderToolTipHeadline) {
		$(self.domSliderToolTipDate).text(dateText);	
	}
	
	if (self.mainController.timeline.markerSpacing == "equal") {
		$("#tl-marker-equal-spacing-date-displayer-"+self.id).text(dateText);
	}
	self.clear3DText();
}

TLMarker.prototype.categoryHasChanged = function() {
	// Here we need to update the text and colour of the marker tab
	// We also need to change the colour of the dot on the timeline scale
	var self = this;
	$(self.domMarkerTab).css({ backgroundColor: "#"+self.category.colour }).text(self.category.title);
	$(self.domBasicInfoHolder).css({ background: "#"+self.category.colour });
	$(self.domColoredPointer).css({ "border-top-color": "#"+self.category.colour });
	$(self.domDurationBar).css({ background: "#"+self.category.colour });
	if (self.timeline.viewType == "category-band") {
		if (self.category.viewType == "duration") {
			$(self.domRoot).removeClass("tl-story-category-view-standard").removeClass("tl-story-category-view-duration").addClass("tl-story-category-view-duration");
		}
		else {
			$(self.domRoot).removeClass("tl-story-category-view-standard").removeClass("tl-story-category-view-duration").addClass("tl-story-category-view-standard");
		}	
	}
	
	self.mainController.sliderController.replacePointOfMarker({marker: self });
	// $(self.domSliderPoint).css({ backgroundColor: "#"+self.category.colour });
	self.clear3DText();
}

TLMarker.prototype.updatePositionFromDate = function() {
	var self = this;
	
	if (self.mainController.timeline.markerSpacing == "equal") {
		// We have to rebuild the entire timeline in this view
		self.mainController.sortMarkersList();

		self.mainController.updateViewsWithNewDateRangeAndZoom({ zoom: self.mainController.timeline.zoom });
	}
	else {
		self.leftOffset = -self.mainController.selectedView.getLeftOffsetForDate({ date: self.startDate });
		self.setMarkerHorizontalAndVerticalPosition();
		self.sliderPointPosition.left = self.mainController.sliderController.getLeftOffsetPercentForDate({ date: self.startDate });
		
		self.mainController.sliderController.replacePointOfMarker({ marker: self });
		
		$(self.domSliderPoint).css({ left: self.sliderPointPosition.left +"%" });
		if (self.startDate.getTime() != self.endDate.getTime() && self.mainController.timeline.viewType == "duration") {
			setTimeout( function() {
				var lineWidth = self.durationBarWidth/self.mainController.selectedView.width * 100;
				$(self.domSliderPoint).css({ width: lineWidth +"%" });
			},1); // Delay needed to get correct durationBarWidth
		} 
		
		self.mainController.sortMarkersList();
		var selectedView = self.mainController.selectedView;
		selectedView.generateMarkerSizeAndPositions();
		selectedView.refreshDisplayMarkers();	
	}
	var timelineDate = theTLSettings.limitDateToRange({ aDate: self.startDate });
	theTLSettings.setCurrentDate({date: timelineDate });
}

TLMarker.prototype.updateDateStatus = function() {
	var self = this;
	self.dateStatus = self.mainController.timeline.storyDateStatus;
	if (self.dateStatus == 1) { // hide
		$(self.domSliderToolTipDate).addClass("tl-ah-hide");
	}
	else if (self.dateStatus == 2) {
		$(self.domSliderToolTipDate).text("~"+self.formatDisplayDate()).removeClass("tl-ah-hide");
	}
	else {
		$(self.domSliderToolTipDate).text(self.formatDisplayDate()).removeClass("tl-ah-hide");
	}

	if (self.dateStatus == 1) { // hide
		$(self.domRoot).addClass("tl-story-block-hide-date");
	}
	else if (self.dateStatus == 2) {
		$(self.domRoot).removeClass("tl-story-block-hide-date");
		$(self.domMarkerDate).text("~"+self.formatDisplayDate());
	}
	else {
		$(self.domRoot).removeClass("tl-story-block-hide-date");
		$(self.domMarkerDate).text(self.formatDisplayDate());
	}
	self.clear3DText();
}


/******************************************************************************************************************************************************************/
TLAdminLightbox.prototype._oldOpenPanel = TLAdminLightbox.prototype.openPanel;
TLAdminLightbox.prototype.openPanel = function(data) {
	var self = this;
	theTLMainController.selected3DView.disableHover = true;
	self._oldOpenPanel();
}

TLAdminLightbox.prototype._oldClosePanel = TLAdminLightbox.prototype.closePanel;
TLAdminLightbox.prototype.closePanel = function(data) {
	var self = this;
	theTLMainController.selected3DView.disableHover = false;
	self._oldClosePanel();
}

