/* Copyright 2013 Webalon Ltd
 *
 * It is strictly forbidden to copy, distribute, modify or reuse this file or its
 * contents without written permission from the copyright holder, Webalon Ltd
 *
 **************************************************************************************/

var TLHashController = function() {
	var self = this;
	self.lastHash = "";
	self.timeout = "";
	self.timeoutTime = 200;
}

TLHashController.prototype = {
	init: function() {
		var self = this;
		theTLSettings.registerAsObserver({ observer: self, type: "currentDate" });
		self.initiateHashCheck();
		return self;	
	},
	setHashToCurrentDate: function() {
		var self = this;
		self.currentDateWasUpdatedTo({ date: AJKHelpers.cloneDate({ date: theTLSettings.currentDate }) });
	},
	setHashToStoryPanel: function(data) {
		var self = this;
		var storyId = data.storyId;
		self.lastHash = "vars!panel="+storyId+"!";
		window.location.hash = self.lastHash;
	},
	currentDateWasUpdatedTo: function(data) {
		var self = this;
		var aDate = data.date;
		var extraInfo = data.extraInfo;
		if (extraInfo && (extraInfo.drag || !extraInfo.updateHash)) {
			// don't update hash on drag or if extrainfo is set and updateHash is false.
			return;	
		}
		self.lastHash = AJKHelpers.formatDate({
			date: aDate,
			formatString: "YYYY-MM-DD_HH:mm:ss",
			forceFullDate: true,
			language: "base"
		});
		self.lastHash = "vars!date="+self.lastHash+"!";
		window.location.hash = self.lastHash;
	},
	initiateHashCheck: function() {
		var self = this;
		self.timeout = setTimeout( function() {
			var newHash = window.location.hash;
			newHash = newHash.replace("#","");
			if (newHash != self.lastHash) {
				//console.log("Hash has changed");
				self.lastHash = newHash;
				self.translateHash( { hash: newHash });
			}
			var thisFunc = arguments.callee;
			self.timeout = setTimeout( function() {
				thisFunc();
			}, self.timeoutTime);
			
		}, self.timeoutTime);
			
	},
	translateHash: function(data) {
		var self = this;
		var storyId = self.getHashPanel({ hash: data.hash });
		if (storyId) {
			theTLMainController.displayPanelForStoryId({ id: storyId });
		}
		else {
			var aDate = self.getHashDate({ hash: data.hash });
			if (aDate) {
				theTLSettings.setCurrentDate({ date: aDate });
				
				// Now hide contentpanel
				theTLMainController.hideContentPanelIfVisible();
			}
		}
	},
	getHashDate: function(data) {
		var self = this;
		var aHash = data.hash;
		aHash = aHash.replace("vars!","!");
		var splitHash = aHash.split("!");
		var aDate = false;
		if (splitHash && splitHash[1]) {
			var info = splitHash[1];
			if (info.indexOf("date=") != -1) {
				var dateTxt = info.split("date=")[1];
				if (dateTxt) {
					dateTxt = dateTxt.replace("_"," ");
					aDate = AJKHelpers.dateFromMySQLDate({ dateString: dateTxt });
				}
			}
		}
		return aDate;
	},
	getHashPanel: function(data) {
		var self = this;
		var aHash = data.hash;
		aHash = aHash.replace("vars!","!");
		var splitHash = aHash.split("!");
		var storyId = false;
		if (splitHash && splitHash[1]) {
			var info = splitHash[1];
			if (info.indexOf("panel=") != -1) {
				var panelTxt = info.split("panel=")[1];
				if (panelTxt) {
					storyId = panelTxt;
				}
			}
		}
		return storyId;
	}
}

var TLSettings = function() {
	var self = this;
	self.observers = new Array();
	self.currentDate = new Date();
	self.visibleStartDate = "";
	self.visibleEndDate = "";
	self.visibleDateRange = ""; // Set in TLSLiderController
	self.windowSize = AJKHelpers.viewportSize();
	self.animateTime = 1000;
	self.onlyMouseScrollIfAlone = true;
	self.lastSelectedStory = "";
	self.flickrApiKey = "1eed7e0b716c8554acfe67108c57952e";
	self.flickrReplacementImage = "/assets/ui/flickr-r2.gif";
}

TLSettings.prototype = {
	sliderDraggerMinWidth: 36,
	init: function() {
		var self = this;
		theAJKWindowResizeEvent.registerAsObserver({ observer: self });
		theAJKMouseScrollEvent.registerAsObserver({ observer: self });
		return self;
	},
	limitDateToRange: function(data) {
		var self = this;
		var aDate = data.aDate;
		aDate = (aDate.getTime() < self.visibleStartDate.getTime()) ? self.visibleStartDate : aDate;
		aDate = (aDate.getTime() > self.visibleEndDate.getTime()) ? self.visibleEndDate : aDate;
		return aDate;
	},
	mouseDidScroll: function(data) {
		var self = this;
		var delta=-data.delta;
		var dateRange = self.visibleEndDate.getTime() - self.visibleStartDate.getTime();
		var change = Math.round(delta/50 * dateRange/2);
		var newDate = new Date();
		newDate.setTime(self.currentDate.getTime()+change);
		newDate = self.limitDateToRange({aDate: newDate});
		if ($.browser.msie) {
			setTimeout( function() {
				self.setCurrentDate({date: newDate, extraInfo: { updateHash: false }});
			},50);			
		}
		else {
			if (!self.testTimeout) {
				self.testTimeout = setTimeout( function() {
					self.setCurrentDate({date: newDate, extraInfo: { updateHash: false, viewOnlySpeed: 1 }});
					self.testTimeout = "";
				},1);
			}
		}
		self.lastSelectedStory = "";	
	},
	windowDidResize: function(data) {
		var self = this;
		self.windowSize = data.newSize;	
	},
	setCurrentDate: function(data) {
		var self = this;
		var date = data.date;
		var extraInfo = data.extraInfo;
		self.currentDate = AJKHelpers.cloneDate({ date: date });
		if (self.observers["currentDate"]) {
			$.each( self.observers["currentDate"], function() {
				if (this.currentDateWasUpdatedTo) {
					this.currentDateWasUpdatedTo({date:self.currentDate, extraInfo: extraInfo });
				}
				
			});
		}
	},
	ensureCurrentDateIsWithinVisibleRange: function() {
		var self = this;
		self.currentDate = self.limitDateToRange({ aDate: self.currentDate }); 
	},
	timeInfo: {
		start: "",
		end: "",
		msecs: "",
		secs: "",
		mins: "",
		hours: "",
		days: "",
		months: "",
		years: "",
		decades: ""
		
	},
	registerAsObserver: function(data) {
		var self = this;
		var type = data.type;
		var observer = data.observer;
		if (!self.observers[type]) {
			self.observers[type] = new Array();	
		}
		self.observers[type] = AJKHelpers.removeItemFromArray({ anArray: self.observers[type], item: observer })
		self.observers[type].push(observer);
	},
	removeAsObserver: function(data) {
		var self = this;	
		var type = data.type;
		var observer = data.observer;
		if (self.observers[type]) {
			self.observers[type] = AJKHelpers.removeItemFromArray({ anArray: self.observers[type], item: observer }); 	
		}
	}
}