/* Copyright 2013 Webalon Ltd
 *
 * It is strictly forbidden to copy, distribute, modify or reuse this file or its
 * contents without written permission from the copyright holder, Webalon Ltd
 *
 **************************************************************************************/

var TL3DViewController = function(data) {
	var self = this;
    if (TLTimelinePrintMode) {
        return self;
    }
	self.domBody = $("body").get()[0];
	self.standardViewController = data.standardViewController;
	self.items = data.items;
	self.timelineStartDate = data.timelineStartDate;
	self.timelineEndDate = data.timelineEndDate;
	self.domCanvas = $("#tl-3d-view-canvas").get()[0];
	self.canvasContext = self.domCanvas.getContext('2d');
	self.timeline3DLength = 1000;
	self.maxTimelineWidth = 0;
	self.controller = data.controller;
	self.sliderController = data.controller.sliderController;
	self.contentPanelController = data.controller.contentPanelController; 
	self.timeline = self.controller.timeline;
	self.timeline3D = self.timeline.timeline3D;
	self.active = false;
	self.viewDateRange = {};
	self.stageScaleRange = {}
	self.visibleMarkers = [];
	self.visibleScalePoints = [];
	self.disableHover = false;
	self.domLaunchButton = $("#tl-timeline-3d-launch").get()[0];
	self.initialised = false;
	self.easeTimeoutTime = 50;
	self.easingFraction = 2;
	self.initialDateSet = false;
	self.viewOffset = 0;
	self.desiredViewOffset = 0;
	self.visibleMarkersLimit = 50;
	self.cachedImageSize = {
		width: 300,
		height: 225	
	}
	self.textImageSize = [{
		width: 300,
		height: 90,
		fontSize: 24,
		lineHeight: 24,
		x: 12,
		y: 46
	},{
		width: 200,
		height: 60,
		fontSize: 16,
		lineHeight: 16,
		x: 8,
		y: 30.667
	},{
		width: 150,
		height: 45,
		fontSize: 12,
		lineHeight: 12,
		x: 6,
		y: 23
	},{
		width: 100,
		height: 30,
		fontSize: 8,
		lineHeight: 8,
		x: 4,
		y: 15.3333
	}]
	self.dateImageSize = [{
		fontSize: 15,
		x: 12,
		y: 22
	},{
		fontSize: 10,
		x: 8,
		y: 14.666667
	},{
		fontSize: 7.5,
		x: 6,
		y: 11
	},{
		fontSize: 5,
		x: 4,
		y: 7.33333
	}]
	self.viewSize = { width: 1, height: 1 };
	self.datePrePadding = 0.05;
	self.timelinePaddingExpansion = 1.2;
	self.moreInfoCutOff = 850;
	self.canvasVert = 0;
	self.initialised = false;

	//self.timeline3D.numCols = 5;
	//self.timeline3D.zoom = 0.25;
	//self.timeline3D.panelSize = 1400;
	//self.timeline3D.vanishTop = 0.1125;
	//self.timeline3D.endToScreenRatio = 1.25; // 0.8-2.0
	//self.timeline3D.direction = 1; // 1=Forward, 0= backward 
	self.version = ++TL3DViewController.counter;
	self.isFireFoxMac = ($.browser.isMac && $.browser.mozilla);
}

TL3DViewController.counter = 0;

TL3DViewController.prototype = {
	init: function() {
		var self = this;

        if (TLTimelinePrintMode) {
            return self;
        }

        self.updateButtonState();
		$(self.domLaunchButton).click( function() {
			if (!self.active) {
				self.launch();	
			}
			else {
				self.close();	
			}
			return false;
		});
		return self;	
	},
	updateAssociations: function() {
		var self = this;	
		self.standardViewController = self.controller.selectedView;
		self.sliderController = self.controller.sliderController;
	},
	initialise: function() {
		var self = this;
		self.initialised = true;

		self.updateStoryColPositions();
				
		self.generateEqualSpacingFakeDates();
		
		/*self.timelinePrePadding = Math.pow(Math.E, -1)/Math.pow(Math.E, -1.1);
		console.log(self.timelinePrePadding);*/
				
		$(self.domCanvas).click( function() {
			$(self.domBody).removeClass("tl-apply-hover-styles");
			if (self.hoverMarker) {
				var ma = self.hoverMarker.marker3DScreenInfo;
				if (ma.timelinePos.y > self.moreInfoCutOff) {
					self.hoverMarker.showExtraInfo();
					self.selectedMarker = "";
					self.hoverMarker = "";
					self.redisplay();
				}
				else {
					var savedHoverMarker = self.hoverMarker; 
					self.controller.focusMarker({ marker: self.hoverMarker }); 
					self.hoverMarker = "";
					self.disableHover = true;
					setTimeout( function() {
						self.selectedMarker = savedHoverMarker;
						self.disableHover = false;
						self.redisplay();
						self.selectedMarker.isEditButtonActive = false;
					},theTLSettings.animateTime+1);
				}
			}
			return false;
		});
	},
	launch: function() {
		var self = this;
		self.active = true;
		if (!self.initialised) {
			self.initialise();
		}
		self.standardViewController.disable();
		self.enable();
	},
	enable: function() {
		var self = this;
		theTLSettings.registerAsObserver({
			type: "currentDate",
			observer: self
		});
		$(self.domBody).addClass("tl-selected-view-3d");
		$(self.domCanvas).css({ display: "block" });
		theAJKMouseMoveEvent.registerAsObserver({ observer: self });

		self.controller.flushSize();
		self.currentDate = AJKHelpers.cloneDate({ date: theTLSettings.currentDate });
		self.updateSlider();
		self.updateButtonState();
		self.redisplayAfterDataChange();
	},
	close: function() {
		var self = this;	
		self.active = false;
		self.disable();
		self.standardViewController.enable();
	},
	disable: function() {
		var self = this;
		theTLSettings.removeAsObserver({ observer: self, type: "currentDate" });
		self.initialDateSet = false;
		$(self.domBody).removeClass("tl-selected-view-3d");
		$(self.domCanvas).css({ display: "none" });
		theAJKMouseMoveEvent.removeAsObserver({ observer: self });
		self.updateSlider();
		self.controller.selectedView.enable();
		self.updateButtonState();
	},
	clearStories3DText: function() {
		var self = this;
		$.each(self.controller.markers, function() {
			this.clear3DText();
		});	
	},	
	updateSlider: function() {
		var self = this;	
		self.sliderController.dragger.updateDraggerSize();
		self.sliderController.calibrateScale();
		self.sliderController.currentDateWasUpdatedTo({ date: theTLSettings.currentDate });
	},
	updateButtonState: function() {
		var self = this;
		if (self.active) {
			$(self.domLaunchButton).text("2d");
		}
		else {
			$(self.domLaunchButton).text("3d");
		}
		if (self.timeline3D.status == 0) {
			$(self.domLaunchButton).css({ display: "none" });
		}
		else {
			$(self.domLaunchButton).css({ display: "block" });
		}
	},
	updateStoryColPositions: function() {
		var self = this;
		
		if (self.timeline.viewType == "category-band") {
			var aCats = self.standardViewController.activeCategories;
			var colInc = 1/aCats.length;
			var cPos = colInc/2;
			var c = 0;
			$.each(aCats, function() {
				this.col3DPos = cPos + c * colInc;
				c++;
			});
			$.each(self.controller.markers, function() {
				this.col3DPos = this.category.col3DPos;
			});
		}
		else {
			var colInc = 1/self.timeline3D.numCols;
			var cPos = 0.5; 
			cPos = (self.timeline3D.numCols % 2) ? cPos : cPos + (colInc/2);
			$.each(self.controller.markers, function() {
				this.col3DPos = cPos;
				cPos+=colInc;
				cPos = (cPos > (1-(colInc/2))) ? cPos-1 : cPos;
			});
		}
	},
	mouseDidMove: function(data) {
		var self = this;
		if (!self.sliderController.dragger.beingDragged && !self.contentPanelController.panelVisible && !self.disableHover) {
			var scr = data.coords;
			// Calculate viewOffset from mousePos
			if (scr.y > self.canvasVert && scr.y < (self.canvasVert+self.viewSize.height) && self.timeline3D.endToScreenRatio > 1) {
				var maxOffset = Math.round(((self.timeline3D.endToScreenRatio-1) * self.viewSize.width)/2*8);
				self.desiredViewOffset = -Math.round((scr.x-(self.viewSize.width/2))/self.viewSize.width/2 * maxOffset);
			}
			else {
				self.desiredViewOffset = 0;
			}
			self.startOffsetEasing();

			
			// Detect hover
			var newHoverMarker = "";
			var scr = data.coords;
			var v2 = self.canvasVert;
			var num = self.visibleMarkers.length;
			for (var c = num-1; c >=0; c--) {
				var m = self.visibleMarkers[c];
				var ma = m.marker3DScreenInfo;
				if (!ma || m.searchHidden) { continue }
				if (ma.active && scr.x > ma.x && scr.x < (ma.x+ma.width) && (scr.y-v2) > ma.y && (scr.y-v2) < (ma.y+ma.height)) { 
					newHoverMarker = m;
					break;
				}
			}
			if (newHoverMarker != self.hoverMarker) {
				if (self.hoverMarker) {
					self.hoverMarker.unfocus();
				}
				self.hoverMarker = newHoverMarker;
				$(self.domBody).removeClass("tl-apply-hover-styles");	
				if (self.hoverMarker) {
					$(self.domBody).addClass("tl-apply-hover-styles");	
				}
				self.redisplay();
			}
			if (self.hoverMarker) {
				self.hoverMarker.focus({ forceUpdate: true });
			}
		}
	},
	mouseDidUp: function() {
		var self = this;
	},
	generateEqualSpacingFakeDates: function() {
		var self = this;
		if (self.timeline.markerSpacing == "equal") {
			$.each(self.controller.markers, function() {
				self.generateFakeDateForStory({ story: this });
			});	
	
			$.each(self.sliderController.scaleDatePoints, function() {
				var viewOffset = this.offset/self.sliderController.sliderScaleWidth * self.standardViewController.width;
				this.fakeDate = self.standardViewController.getDateFromLeftOffset({ leftOffset: viewOffset });
			});
		}
	},
	generateFakeDateForStory: function(data) {
		var self = this;
		var story = data.story;
		story.fakeStartDate = self.standardViewController.getDateFromLeftOffset({ leftOffset: story.leftOffset });
	},
	currentDateWasUpdatedTo: function(data) {
		var self = this;
		var animate = (data.extraInfo) ? data.extraInfo.animate : false;
		if (!self.initialDateSet && !animate) {
			self.initialDateSet=true;
			self.moveViewToDate({date: data.date });
		}
		else if (animate) {
			self.animateViewToDate({date: data.date, speed: theTLSettings.animateTime });
		}
		else {
			self.easeViewToDate({date: data.date });
		}
		self.standardViewController.updateDisplayDate({ date: data.date });
	},
	animateViewToDate: function(data) {
		var self = this;
		var speed = data.speed;
		var sTime = self.currentDate.getTime();
		var eTime = data.date.getTime();
		
		if (self.animationTimeout) {
			clearTimeout(self.animationTimeout);
			self.animationTimeout = "";	
		}
		$({ time: sTime }).animate({
			time: eTime	
		},{
			duration: speed, 
			step: function() {
				var cTime = new Date();
				cTime.setTime(this.time);
				self.moveViewToDate({ date: cTime });
			},
			complete: function() {}
		});

	},
	easeViewToDate: function(data) {
		var self = this;
		self.desiredDate = AJKHelpers.cloneDate({ date: data.date });
		var instantly = data.instantly;
		if (instantly) {
			self.currentDate = AJKHelpers.cloneDate({ date: self.desiredDate }); 
		} 
		self.startStageEasing();
	},
	startStageEasing: function() {
		var self = this;	
		if (!self.animationTimeout) {
			(function() {
				if (self.currentlyAnimating) {
					return;	
				}
				var newDateTime = self.currentDate.getTime() + Math.round((self.desiredDate.getTime()-self.currentDate.getTime())/self.easingFraction); 
				var newDate = new Date();
				newDate.setTime(newDateTime);
				if (Math.abs(newDateTime - self.desiredDate.getTime()) > 1000 || self.desiredViewOffset-self.viewOffset >= 0.5) {
					var thisFunc = arguments.callee;
					self.animationTimeout = setTimeout(function() {
						thisFunc();
					},self.easeTimeoutTime);
					self.viewOffset+=(self.desiredViewOffset-self.viewOffset)/2;
					self.moveViewToDate({ date: newDate });
				} 
				else {
					self.viewOffset = self.desiredViewOffset;
					self.moveViewToDate({ date: self.desiredDate });
					self.animationTimeout = "";
				}				
			})();	
		}
	},
	startOffsetEasing: function() {
		var self = this;	
		if (!self.offsetAnimation) {
			(function() {
				if (self.currentlyAnimating) {
					return;	
				}
				if (Math.abs(self.desiredViewOffset-self.viewOffset) >= 1) {
					var thisFunc = arguments.callee;
					self.offsetAnimation = setTimeout(function() {
						thisFunc();
					},self.easeTimeoutTime);
					self.viewOffset+=(self.desiredViewOffset-self.viewOffset)/4;
					self.drawView();
				} 
				else {
					self.viewOffset = self.desiredViewOffset;
					self.drawView();
					self.offsetAnimation = "";
				}
				if (self.hoverMarker && theTLMainController.adminController) {
					self.hoverMarker.focus({ forceUpdate: true });
				}
			
			})();	
		}
	},
	moveViewToDate: function(data) {
		var self = this;
		self.selectedMarker = "";
		var cDate = AJKHelpers.cloneDate({ date: data.date });
		self.currentDate = cDate;
		
		var datePrePaddingInMS = parseInt(theTLSettings.visibleDateRange*self.timeline3D.zoom*self.datePrePadding,10);
		var dateRange = parseInt(theTLSettings.visibleDateRange*self.timeline3D.zoom,10)+datePrePaddingInMS;
		
		self.viewDateRange.end = new Date();
		self.viewDateRange.start = new Date(); 
		self.viewDateRange.range = dateRange;
		
		if (self.timeline3D.direction) {
			self.viewDateRange.start.setTime(cDate.getTime()-datePrePaddingInMS);
			self.viewDateRange.end.setTime(self.viewDateRange.start.getTime()+dateRange);
		}
		else { 
			self.viewDateRange.end.setTime(cDate.getTime()+datePrePaddingInMS);
			self.viewDateRange.start.setTime(self.viewDateRange.end.getTime()-dateRange);
		}
		
		self.visibleScalePoints = [];
		$.each(self.sliderController.scaleDatePoints, function() {
			var thisScalePt = this;
			var thisDate = (self.timeline.markerSpacing == "equal") ? this.fakeDate : this.date;
			if (thisDate.getTime() >= self.viewDateRange.start.getTime() && thisDate.getTime() <= self.viewDateRange.end.getTime()) {
				self.visibleScalePoints.push(thisScalePt);
			}
		});
		
		self.visibleMarkers = [];
		$.each(self.controller.markers, function() {
			var thisMarker = this;
			var startDate = this.startDate;
			if (self.timeline.markerSpacing == "equal") {
				if (!this.fakeStartDate) {
					self.generateFakeDateForStory({ story: this });	
				}
				startDate = this.fakeStartDate;
			}
			if (startDate.getTime() >= self.viewDateRange.start.getTime() && startDate.getTime() <= self.viewDateRange.end.getTime()) {
				if (self.timeline3D.direction) {
					self.visibleMarkers.unshift(thisMarker);
				}
				else {
					self.visibleMarkers.push(thisMarker);
				}
			}
		});
		var numVis = self.visibleMarkers.length; 
		if (numVis > self.visibleMarkersLimit) {
			self.visibleMarkers = self.visibleMarkers.slice(numVis-self.visibleMarkersLimit)
		}
		self.drawView();
	},
	resizeContent: function(data) {
		var self = this;
		self.viewSize = {
			width: data.newSize.width,
			height: data.newSize.height
		}
		self.maxTimelineWidth = self.viewSize.width*self.timeline3D.endToScreenRatio*self.timelinePaddingExpansion;
		$(self.domCanvas).css({ width: self.viewSize.width, height: self.viewSize.height }).attr("width",self.viewSize.width).attr("height",self.viewSize.height);
		self.canvasVert = AJKHelpers.getCoordsOfDomEl({ domEl: self.domCanvas }).y;
		self.drawView();
	},
	getScreenInfoFromTimelinePos: function(data) {
		var self = this;
		var timelinePos = data.timelinePos;
		var screenWidth = self.viewSize.width;
		var screenHeight = Math.round(self.viewSize.height*self.timelinePaddingExpansion);
		var pos = {
			top: screenHeight * self.timeline3D.vanishTop,
			topLeft: screenWidth * 0.5,
			topRight: screenWidth * 0.5,
			bottom: screenHeight * 1,
			bottomLeft: -screenWidth * .25 * self.timelinePaddingExpansion,
			bottomRight: screenWidth * self.timeline3D.endToScreenRatio * self.timelinePaddingExpansion
		}
		var distance = timelinePos.y;
		var hOffset = timelinePos.x;
	
		var itemWidth = pos.bottomRight-pos.bottomLeft;
		var vanishingHeight = pos.bottom-pos.top;
		var maxDistance = self.timeline3DLength;
		var step = 20;
		var k = 0.006;

		var widthOnScreen = itemWidth * Math.pow(Math.E, -k * (maxDistance - distance));
		
		var xPos = (screenWidth-widthOnScreen)/2+hOffset*widthOnScreen;
		var yPos = pos.top+(widthOnScreen/itemWidth) * vanishingHeight;
		var offset = Math.min(widthOnScreen/itemWidth * self.viewOffset);
		return {
			x: xPos+offset,
			y: yPos-10,
			sliceWidth: widthOnScreen,
			offset: offset
		}

	},
	redisplayAfterDataChange: function() {
		var self = this;
		if (self.active) {
			self.updateStoryColPositions();
			self.moveViewToDate({ date: self.currentDate });
		}
	},
	redisplay: function() {
		var self = this;
		if (self.active) {
			self.drawView();
		}
	},
	drawView: function() {
		var self = this;
		var catView = (self.timeline.viewType == "category-band");
		var numCols = (catView) ? self.standardViewController.activeCategories.length : self.timeline3D.numCols;
		var ctx = self.canvasContext;
		var vanishingPoint = self.getScreenInfoFromTimelinePos({ timelinePos: { x: 0.5, y: 0 } });

		var tcRGB = AJKHelpers.convertHexColourToRGB({ hexColour: self.timeline3D.color });
		var timelineGradColor = "rgba("+tcRGB.r+","+tcRGB.g+","+tcRGB.b+",";

		ctx.clearRect(0,0,self.viewSize.width,self.viewSize.height);
		
		var bgRGB = AJKHelpers.convertHexColourToRGB({ hexColour: self.timeline.backgroundColour });
		var bgFade = "rgba("+bgRGB.r+","+bgRGB.g+","+bgRGB.b+","+self.timeline3D.bgFade+")";
		ctx.fillStyle=bgFade;
		ctx.fillRect(0,0,self.viewSize.width,self.viewSize.height);
		
		// Back gradient
		var leftBottomPos = self.getScreenInfoFromTimelinePos({ timelinePos: { x: 0, y: self.timeline3DLength } });
		var rightBottomPos = self.getScreenInfoFromTimelinePos({ timelinePos: { x: 1, y: self.timeline3DLength } });
		if (catView) {
			for (var cat = 0; cat<numCols; cat++) {
				var thisCat = self.standardViewController.activeCategories[cat];
				var catRGB = AJKHelpers.convertHexColourToRGB({ hexColour: thisCat.colour });
				var catGradColor = "rgba("+catRGB.r+","+catRGB.g+","+catRGB.b+",";
				var blPos = self.getScreenInfoFromTimelinePos({ timelinePos: { x: cat/numCols, y: self.timeline3DLength } });
				var brPos = self.getScreenInfoFromTimelinePos({ timelinePos: { x: (cat+1)/numCols, y: self.timeline3DLength } });
				
				var grad= ctx.createLinearGradient(0,self.viewSize.height*self.timeline3D.vanishTop,0, self.viewSize.height);
				grad.addColorStop(0, catGradColor+"0)");
				grad.addColorStop(0.06, catGradColor+"0)");
				grad.addColorStop(0.8, catGradColor+"1)");
				grad.addColorStop(1, catGradColor+"1)");
				ctx.fillStyle = grad;
				ctx.beginPath();
				ctx.moveTo(vanishingPoint.x,vanishingPoint.y);
				ctx.lineTo(blPos.x, blPos.y);
				ctx.lineTo(brPos.x, brPos.y);
				ctx.moveTo(vanishingPoint.x,vanishingPoint.y);
				ctx.closePath();
				ctx.fill();			
			}
			
		}
		else {		
			var grad= ctx.createLinearGradient(0,self.viewSize.height*self.timeline3D.vanishTop,0, self.viewSize.height);
			grad.addColorStop(0, timelineGradColor+"0)");
			grad.addColorStop(0.06, timelineGradColor+"0)");
			grad.addColorStop(0.8, timelineGradColor+"0.75)");
			grad.addColorStop(1, timelineGradColor+"0.75)");
			ctx.fillStyle = grad;
			ctx.beginPath();
			ctx.moveTo(vanishingPoint.x,vanishingPoint.y);
			ctx.lineTo(leftBottomPos.x, leftBottomPos.y);
			ctx.lineTo(rightBottomPos.x, rightBottomPos.y);
			ctx.lineTo(vanishingPoint.x,vanishingPoint.y);
			ctx.closePath();
			ctx.fill();
		}

		// Column lines
		for (var endPt = 0; endPt<=numCols; endPt++) {
			var endPos = self.getScreenInfoFromTimelinePos({ timelinePos: { x: endPt/numCols, y: self.timeline3DLength } });
			ctx.beginPath();
			var grad= ctx.createLinearGradient(vanishingPoint.x,vanishingPoint.y,endPos.x, endPos.y);
			grad.addColorStop(0, timelineGradColor+"0)");
			grad.addColorStop(0.06, timelineGradColor+"0)");
			grad.addColorStop(0.8, timelineGradColor+"1)");
			grad.addColorStop(1, timelineGradColor+"1)");
			ctx.strokeStyle = grad;
			ctx.moveTo(vanishingPoint.x,vanishingPoint.y);
			ctx.lineTo(endPos.x, endPos.y);
			ctx.closePath();
			ctx.stroke();			
		}
		
		// Date points
		ctx.beginPath();
		var grad = ctx.createLinearGradient(0,self.viewSize.height*self.timeline3D.vanishTop,0, self.viewSize.height);
		grad.addColorStop(0, timelineGradColor+"0)");
		grad.addColorStop(0.06, timelineGradColor+"0)");
		grad.addColorStop(0.8, timelineGradColor+"0.75)");
		grad.addColorStop(1, timelineGradColor+"0.75)");
		ctx.strokeStyle = grad;
		
		var textGrad = ctx.createLinearGradient(0,self.viewSize.height*self.timeline3D.vanishTop,0, self.viewSize.height);
		textGrad.addColorStop(0, timelineGradColor+"0)");
		textGrad.addColorStop(0.06, timelineGradColor+"0)");
		textGrad.addColorStop(0.6, timelineGradColor+"0.9)");
		textGrad.addColorStop(1, timelineGradColor+"1)");
		ctx.fillStyle = (self.isFireFoxMac) ? timelineGradColor+"0.7)" : textGrad;

		$.each( self.visibleScalePoints, function() {
			var thisScalePoint = this;
			var thisDate = (self.timeline.markerSpacing == "equal") ? this.fakeDate : this.date;

			if (self.timeline3D.direction) {
				var yPos = (self.viewDateRange.end.getTime()-thisDate.getTime())/self.viewDateRange.range * self.timeline3DLength;
			}
			else {
				var yPos = (thisDate.getTime()-self.viewDateRange.start.getTime())/self.viewDateRange.range * self.timeline3DLength;
			}

			self.drawScaleOnScreenAtTimelinePos({ 
				timelinePos: { y: yPos },
				scalePoint: thisScalePoint 
			});
			
		});
		ctx.closePath();
		ctx.stroke();			
		ctx.fill();			

		// Markers
		$.each( self.visibleMarkers, function() {
			var thisMarker = this;
			var xPos = this.col3DPos;
			var startDate = (self.timeline.markerSpacing == "equal") ? this.fakeStartDate : this.startDate;
			
			if (self.timeline3D.direction) {
				var yPos = (self.viewDateRange.end.getTime()-startDate.getTime())/self.viewDateRange.range * self.timeline3DLength;
			}
			else {
				var yPos = (startDate.getTime()-self.viewDateRange.start.getTime())/self.viewDateRange.range * self.timeline3DLength;
			}
			self.drawMarkerOnScreenAtTimelinePos({ 
				timelinePos: {x: xPos, y: yPos },
				marker: thisMarker 
			});
			
		});
	},
	drawScaleOnScreenAtTimelinePos: function(data) {
		var self = this;	
		var ctx = self.canvasContext;
		var timelinePos = data.timelinePos;
		timelinePos.x = 0;
		var scalePoint = data.scalePoint;
		var screenPos = self.getScreenInfoFromTimelinePos({ timelinePos: timelinePos });
		screenPos.y = Math.round(screenPos.y);
		var scaleWidth = screenPos.sliceWidth*1.05;
		var extendWidth = (scaleWidth-screenPos.sliceWidth)/2; 
		if (scaleWidth > 100) {
			var scaleLeft = (self.viewSize.width-scaleWidth)/2+screenPos.offset;
			ctx.moveTo(scaleLeft,screenPos.y);
			if (scalePoint.dateText) {
				ctx.lineTo(scaleLeft+scaleWidth, screenPos.y);
			}
			else {
				ctx.lineTo(scaleLeft+extendWidth, screenPos.y);
				ctx.moveTo(scaleLeft+extendWidth+screenPos.sliceWidth,screenPos.y);
				ctx.lineTo(scaleLeft+scaleWidth, screenPos.y);
			}
			var fontSize = scaleWidth/30;
			ctx.font = fontSize+"px 'Helvetica','sans-serif'";
			ctx.textAlign = "right";
			ctx.fillText(scalePoint.dateText, scaleLeft-(extendWidth/2), screenPos.y+0.25 * fontSize-1);
			ctx.textAlign = "left";
			ctx.fillText(scalePoint.dateText, scaleLeft+scaleWidth+(extendWidth/2), screenPos.y+0.25 * fontSize-1);

		}
	},
	drawMarkerOnScreenAtTimelinePos: function(data) {
		var self = this;
		var ctx = self.canvasContext;

		var timelinePos = data.timelinePos;
		var marker = data.marker;
		
		if (marker.category.hide) {
			//dont't display stories that have their category hidden
			return;	
		}
		
		var vPos = timelinePos.y;

		if (self.timeline.viewType == "color-category-stories" || self.timeline.viewType == "duration") {
			var panelColor = "#"+marker.category.colour;
			var cRGB = AJKHelpers.convertHexColourToRGB({ hexColour: marker.category.colour });
			var gradColor = "rgba("+cRGB.r+","+cRGB.g+","+cRGB.b+",";
		}
		else {
			var panelColor = "rgba(255,255,255,1)";
			var gradColor = "rgba(255,255,255,";
		}
		
		var screenPos = self.getScreenInfoFromTimelinePos({ timelinePos: timelinePos });
		var sliceWidth = screenPos.sliceWidth;
		var sizeAdjust = sliceWidth/self.timeline3D.panelSize;
		
		// Need to work this our on basis of how far away it is
		var itemWidth = 200 * sizeAdjust;

		if (itemWidth < 3) { return }
        
		var textHolderHeight = 60 * sizeAdjust;
		var imageBoxHeight = 150 * sizeAdjust;
		var arrowHeight = 15 * sizeAdjust;
		var arrowWidth = 15 * sizeAdjust;
		var itemHeight = textHolderHeight+imageBoxHeight+arrowHeight;
		var itemHeightNoImage = textHolderHeight+arrowHeight;
		var textPadding = itemWidth/30;
		var headlineSize = 15 * sizeAdjust;
		var maxMaxTextWidth = itemWidth - textPadding * 2; 
		var vShift = (self.hoverMarker == marker) ? 5 * sizeAdjust : 0;
		

            marker.marker3DScreenInfo = {
			x: screenPos.x-itemWidth/2,
			y: screenPos.y-itemHeightNoImage,
			width: itemWidth,
			height: itemHeightNoImage,
			active: true,
			timelinePos: timelinePos
		}

		var fadeOutLimit = self.timeline3DLength-(self.datePrePadding/(self.datePrePadding+1) * self.timeline3DLength); 
		if (vPos > fadeOutLimit) {
			opacity = 1-(vPos - fadeOutLimit)/(self.timeline3DLength-fadeOutLimit);
			marker.marker3DScreenInfo.active = (opacity>0.6) ? true : false; 
		}
		else {
			var opacity = sliceWidth*self.timelinePaddingExpansion/(0.3*self.maxTimelineWidth);
		}
		
		//opacity *= opacity;
		opacity = (opacity > 1 || (self.hoverMarker == marker)) ? 1 : opacity;
		
		opacity = (marker.searchHidden) ? opacity * 0.1 : opacity;
		
		ctx.globalAlpha = opacity;

		var startPos = { y: screenPos.y-vShift, x: screenPos.x }

		// Draw Text Box
		ctx.fillStyle = panelColor;
		ctx.beginPath();
		ctx.moveTo(startPos.x,startPos.y);
		startPos.x-=arrowWidth/2;
		startPos.y-=arrowHeight;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.x-=(itemWidth/2-arrowWidth/2);
		ctx.lineTo(startPos.x,startPos.y);
		startPos.y-=textHolderHeight;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.x+=itemWidth;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.y+=textHolderHeight;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.x-=(itemWidth/2-arrowWidth/2);
		ctx.lineTo(startPos.x,startPos.y);
		ctx.lineTo(screenPos.x,screenPos.y-vShift);
		ctx.closePath();
		ctx.fill();
		
		//Redraw arrow 
		startPos = { y: screenPos.y-vShift, x: screenPos.x }
		ctx.beginPath();
		ctx.moveTo(startPos.x,startPos.y);
		startPos.x-=arrowWidth/2;
		startPos.y-=arrowHeight;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.x+=arrowWidth;
		ctx.lineTo(startPos.x,startPos.y);
		ctx.closePath();
		ctx.fill();

		// Draw text
        if (!marker.dom3DText) {
            marker.generateTextImageFor3D();
        }
        var bestDomText = "";
        if (itemWidth <= 75) {
            bestDomText = marker.dom3DTextTiny;
        }
        else if (itemWidth <= 150) {
            bestDomText = marker.dom3DTextSmall;
        }
        else if (itemWidth <= 200) {
            bestDomText = marker.dom3DText;
        }
        else {
            bestDomText = marker.dom3DText;
        }
        ctx.drawImage(bestDomText,screenPos.x-0.5 * itemWidth, screenPos.y-arrowHeight-textHolderHeight-vShift,itemWidth,textHolderHeight);
		
		// Draw image
		if (marker.feedSource == "twitter") {
		}
		else if (!marker.lazyImageLoaded && marker.displayImage) {
			marker.lazyLoadImage();
		}
		else if (marker.displayImage && marker.domMainImage && itemWidth>1) {
			if (!marker.dom3DImage) {
				marker.generateImageFor3D();
			}
			if (marker.dom3DImage) {
				//ctx.fillStyle = "rgba(0,0,0,1)";
				//ctx.fillRect(screenPos.x-0.5 * itemWidth, screenPos.y-itemHeight,itemWidth,imageBoxHeight);
				var bestDomImage = "";
				if (itemWidth <= 100) {
					bestDomImage = marker.dom3DImage100;
				}
				else if (itemWidth <= 150) {
					bestDomImage = marker.dom3DImage150;
				}
				else if (itemWidth <= 200) {
					bestDomImage = marker.dom3DImage200;
				}
				else {
					bestDomImage = marker.dom3DImage;
				}
                
				ctx.drawImage(bestDomImage,screenPos.x-0.5 * itemWidth, screenPos.y-itemHeight-vShift,itemWidth,imageBoxHeight);
				marker.marker3DScreenInfo.height = itemHeight;
				marker.marker3DScreenInfo.y = screenPos.y-itemHeight;
			}
		}
		
		if ((self.selectedMarker == marker || self.hoverMarker == marker) && marker.marker3DScreenInfo.timelinePos.y > self.moreInfoCutOff) {
			var clickBlock = {
				height: 20 * sizeAdjust,
				fontSize: 11 * sizeAdjust,
				bottom: 6 * sizeAdjust,
				left: 10 * sizeAdjust
			}
			ctx.fillStyle = "rgba(0,0,0,0.75)";
			var blockX = screenPos.x-0.5*itemWidth;
			var blockY = screenPos.y-itemHeightNoImage-vShift-clickBlock.height;
			ctx.fillRect(blockX, blockY,itemWidth,clickBlock.height);		
			ctx.fillStyle = "rgba(255,255,255,0.8)";
			ctx.font = clickBlock.fontSize+"px 'Helvetica','sans-serif'";
			ctx.textAlign = "left";
			ctx.fillText("CLICK TO FIND OUT MORE", blockX+clickBlock.left, blockY+(clickBlock.height)-clickBlock.bottom);

		}
		
		// Draw reflection
		var startPos = {
			y: screenPos.y+vShift,
			x: screenPos.x
		}

		var grad= ctx.createLinearGradient(screenPos.x,screenPos.y,screenPos.x, screenPos.y+textHolderHeight+arrowHeight);
		grad.addColorStop(0, gradColor+"0.5)");
		grad.addColorStop(0.3, gradColor+"0.2)");
		grad.addColorStop(0.7, gradColor+"0.05)");
		grad.addColorStop(1, gradColor+"0)");
		ctx.fillStyle = grad;
		ctx.beginPath();
		ctx.moveTo(startPos.x,startPos.y);
		startPos.x+=arrowWidth/2;
		startPos.y+=arrowHeight;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.x+=(itemWidth/2-arrowWidth/2);
		ctx.lineTo(startPos.x,startPos.y);
		startPos.y+=textHolderHeight;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.x-=itemWidth;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.y-=textHolderHeight;
		ctx.lineTo(startPos.x,startPos.y);
		startPos.x+=(itemWidth/2-arrowWidth/2);
		ctx.lineTo(startPos.x,startPos.y);
		ctx.lineTo(screenPos.x,screenPos.y+vShift);
		ctx.closePath();
		ctx.fill();
		ctx.globalAlpha = 1;
	}
}


var TLViewController = function(data) {
	var self = this;
	self.items = data.items;
	self.timeline = data.timeline;
	self.domEl = data.domEl;
	self.zoom = data.zoom;
	self.timeline = data.timeline;
	self.currentViewType = self.timeline.viewType;
	self.type =  self.zoom;
	self.domDateDisplayer = data.domDateDisplayer;
	self.originalStartDate = data.originalStartDate;
	self.originalEndDate = data.originalEndDate;
	self.startDate = data.startDate;
	self.endDate = data.endDate;
	self.dateRange = this.endDate.getTime()-this.startDate.getTime();
	self.scaleColour = data.scaleColour;
	self.viewScaleBlocks = new Array();
	self.domViewBlocksHolder = "";
	self.width = 0;
	self.stageDraggedFunc = data.stageDraggedFunc;
	self.stageLeftOffset = "";
	self.realStageLeftOffset = "";
	self.animationTimeout = "";
	self.currentlyAnimating = false;
	self.viewScaleSettings = new Array();
	self.minStagePos = 0;
	self.maxStagePos = 0;
	self.domStageStoryHolder = "";
	self.minIdealStorySeparation = (self.timeline.viewType == "category-band") ? 200 : 327;
	self.availableScales = new Array();
	self.availableScalesByName = new Array();
	self.draggableObj = "";
	self.displayDate = "";
	self.easeTimeoutTime = 50;
	self.easingFraction = 2;
	self.categoryBands = [];
	self.categoryBandsByKey = [];
}

// Marker spacinf view stuff

TLViewController.markerSpacingView = [{
		type: "standard",
		markerSpacing: 0,
		categoryViewMarkerSpacing: 0
	}, {
		type: "equal",
		markerSpacing: 352,
		categoryViewMarkerSpacing: 238
	}, {
		type: "equal",
		markerSpacing: 176,
		categoryViewMarkerSpacing: 300
	},{
		type: "top-to-bottom",
		rows: 3
	},{
		type: "top-to-bottom",
		rows: 4
	},{
		type: "top-to-bottom",
		rows: 5
	},{
		type: "top-to-bottom",
		rows: 6
	},{
		type: "top-to-bottom",
		rows: 7
	},{
		type: "top-to-bottom",
		rows: 8
	},{
		type: "top-to-bottom",
		rows: 9
	},{
		type: "top-to-bottom",
		rows: 10
	}
]
// Class level funcs

TLViewController.isASuitableScaleForDateRange = function(data) {
	var scaleName = data.scaleName;
	var startDate = data.startDate;
	var endDate = data.endDate;
	var availableScales = TLViewController.getAvailableScalesForDateRange({
		startDate: startDate,
		endDate: endDate	
	});
	var matchFound = false;
	$.each( availableScales, function() {
		if (this.name == scaleName) {
			matchFound = true;	
		}
	});
	return matchFound;
}

TLViewController.getBestZoomForDateRange = function(data) {
	var startDate = data.startDate;
	var endDate = data.endDate;
	var availableScales = TLViewController.getAvailableScalesForDateRange({
		startDate: startDate,
		endDate: endDate	
	});

	var idealWidth = 30000;
	var scaleClosestToIdealWidth = "";

	$.each(availableScales, function() {
		var scaleWidth = (endDate.getTime()-startDate.getTime()) * this.object.getStageWidthRatio();
		var distanceFromIdeal = Math.abs(scaleWidth-idealWidth);

		if (!scaleClosestToIdealWidth || (distanceFromIdeal < scaleClosestToIdealWidth.distanceFromIdeal)) {
			scaleClosestToIdealWidth = this;
			scaleClosestToIdealWidth.distanceFromIdeal = distanceFromIdeal;
		}
	});
		
	return scaleClosestToIdealWidth;
}
	
TLViewController.getAvailableScalesForDateRange = function(data) {
	var startDate = data.startDate;
	var endDate = data.endDate;
	var scale = "";
	var availableScales = new Array();
	var availableScalesByName = new Array();
	var allScales = new Array();
	for (scale in TLViewController.viewScaleSettings) {
		var thisScale = TLViewController.viewScaleSettings[scale];
		if (scale != "indexOf") {
			var scaleWidth = (endDate.getTime()-startDate.getTime()) * thisScale.getStageWidthRatio();
			
			var minWidth = 500;
			var maxWidth = 500000;
			
			var scaleInfoObj = {
				name: scale,
				width: scaleWidth,
				object: thisScale
			}

			if (scaleWidth <= maxWidth && scaleWidth >= minWidth) {
				availableScales.push(scaleInfoObj);
				availableScalesByName[scaleInfoObj.name] = scaleInfoObj;
			}
	
			allScales.push(scaleInfoObj);
		}
	}

	availableScales.sort(function(a,b) {
		return (a.width > b.width) ? 1 : (a.width==b.width) ? 0 : -1;
	});
	
	if (availableScales.length == 0) {
		var smallestScale = "hour-1-mins-large";
		var largestScale = "large-date-range-9";
		if (endDate.getTime()-startDate.getTime() < AJKHelpers.dateOneDayInMS) {
			var aScale = {
				name: smallestScale,
				width: 1,
				object: TLViewController.viewScaleSettings[smallestScale]
			}
		}
		else {
			var aScale = {
				name: largestScale,
				width: 1,
				object: TLViewController.viewScaleSettings[largestScale]
			}
		}
		availableScales.push(aScale);
	}

	/*allScales.sort(function(a,b) {
		return (a.width > b.width) ? 1 : (a.width==b.width) ? 0 : -1;
	});
	
	var sampleScales = "";
	$.each(self.allScales, function() {
		sampleScales += this.name+" "+this.width+"\n";
	});
	alert(sampleScales);*/

	return availableScales;
}

/* Instance level classes */

TLViewController.prototype = {
	init: function() {
		var self = this;
		
		theTLSettings.registerAsObserver({
			type: "currentDate",
			observer: self
		});
		$(self.domEl).addClass("tl-stage-view-"+self.currentViewType);
		if (self.currentViewType == "duration") {
			$(self.domEl).addClass("tl-stage-view-color-category-stories");
		}

		if (self.timeline.viewType == "category-band") {
		 	self.timeline.equalMarkerSpacing = TLViewController.markerSpacingView[self.timeline.storySpacingType].categoryViewMarkerSpacing;
		}
		else {
		 	self.timeline.equalMarkerSpacing = TLViewController.markerSpacingView[self.timeline.storySpacingType].markerSpacing;
		}

		
		$(theTLMainController.domFixedStageContent).find(".tl-view-category-band-label").remove();

		self.easeTimeoutTime = ($.browser.msie) ? 75 : 50;
		//self.easeTimeoutTime = ($.browser.mozilla && !$.browser.isMac) ? 125 : self.easeTimeoutTime;
		self.easingFraction = ($.browser.msie) ? 1.75 : 2;
		//self.easingFraction = ($.browser.mozilla && !$.browser.isMac) ? 1.5 : self.easingFraction;

		theAJKWindowResizeEvent.registerAsObserver({ observer: self });

		// Check that the selectedView is suitable for the timeline.
		// If not, set it to one of the suitable views
		
		self.availableScales = TLViewController.getAvailableScalesForDateRange({
			startDate: self.originalStartDate,
			endDate: self.originalEndDate
		});

		/*self.availableScales = TLViewController.getAvailableScalesForDateRange({
			startDate: theTLSettings.timeInfo.start,
			endDate: theTLSettings.timeInfo.end
		});*/
		
		/*if (scaleClosestToIdealWidth && !self.availableScalesByName[self.type]) {
			self.type = scaleClosestToIdealWidth.name;
		}*/
				
		$(self.domEl).addClass("tl-stage-"+self.type);
		$(self.domEl).addClass("tl-stage-view-"+self.currentViewType);
		
		if (self.timeline.markerSpacing == "equal") { // equal spacing view
			self.timeline.equalMarkerLeftPadding = self.timeline.initialEqualMarkerLeftPadding * self.items.length /140 * self.timeline.equalMarkerSpacing/176;
			self.timeline.equalMarkerTotalPadding = self.timeline.initialEqualMarkerTotalPadding * self.items.length /140 * self.timeline.equalMarkerSpacing/176; 
			self.timeline.equalMarkerLeftPadding = (self.timeline.equalMarkerLeftPadding < 500) ? 500 : self.timeline.equalMarkerLeftPadding;
			self.timeline.equalMarkerTotalPadding = (self.timeline.equalMarkerTotalPadding < 750) ? 750 : self.timeline.equalMarkerTotalPadding;
			self.width = self.timeline.equalMarkerTotalPadding + self.items.length * self.timeline.equalMarkerSpacing;
			self.width = (self.width < self.timeline.equalSpacingMinWidth) ? self.timeline.equalSpacingMinWidth : self.width;
		}
		else { // default view
			// Need to change this depending on the view
			self.width = (theTLSettings.timeInfo.end.getTime()-theTLSettings.timeInfo.start.getTime()) * TLViewController.viewScaleSettings[self.type].getStageWidthRatio();
		}
		
		$(self.domEl).empty();
		$(self.domEl).css({ width: self.width });
		if (self.timeline.viewType == "category-band") {
			self.createCategoryBands();
		}
		else {
			self.createViewScaleBlocks();
		}
		self.displayMarkers();
		self.initialiseViewDrag();
		
		$(self.domEl).click( function(e) {

			var domStoryBlock = AJKHelpers.getSelfOrFirstParantOfClass({
				domEl: e.target,
				className: "tl-story-block"
			});
			
			if (domStoryBlock) {
				var key = $(domStoryBlock).attr("markerKey");
				var clickedStoryBlock = theTLMainController.markersByKey[key];
				
				if ($(e.target).hasClass("ig-inner-image-mask") && clickedStoryBlock.isFlickrImage) {
					if (!AJKDraggable.cancelClick) {
						var flickrImagePage = AJKHelpers.getFKRPhotoPageFromImageSrc({ src: clickedStoryBlock.displayImage.src});
						window.open(flickrImagePage);
					}
				}
				else if ($(e.target).hasClass("tl-sb-more-button") || self.timeline.viewType == "duration" || (self.timeline.viewType == "category-band" && clickedStoryBlock.category.viewType == "duration")) {
					clickedStoryBlock.showExtraInfo();					
				}
				else {
					theTLMainController.hideContentPanelIfVisible();
				}
			}
			else {
				theTLMainController.hideContentPanelIfVisible();
			}
			return false;
		})
		.mouseover( function(e) {
			var domImageGalleryBlock = AJKHelpers.getSelfOrFirstParantOfClass({
				domEl: e.target,
				className: "tl-story-block"
			});
			if (domImageGalleryBlock) {
				var key = $(domImageGalleryBlock).attr("markerKey");
				var relatedStory = theTLMainController.markersByKey[key];
				relatedStory.focus();
			}
		})
		.mouseout( function(e) {
			var domImageGalleryBlock = AJKHelpers.getSelfOrFirstParantOfClass({
				domEl: e.target,
				className: "tl-story-block"
			});
			// Check that the target of the mouseout was the image gallery
			if (domImageGalleryBlock) {
				var key = $(domImageGalleryBlock).attr("markerKey");
				// Check that the element the mouse went to is not within tl-sb-image-gallery and if it is check
				// thet the galery el is for another story by comparing keys
				var relTarg = e.relatedTarget || e.toElement;
				var toStoryBlock = AJKHelpers.getSelfOrFirstParantOfClass({ domEl: relTarg, className: "tl-story-block"});
				var toKey = (toStoryBlock) ? $(toStoryBlock).attr("markerKey") : "";   
				if (!toStoryBlock || toKey != key) {
					var relatedStory = theTLMainController.markersByKey[key];
					relatedStory.unfocus();
				}
			}
		});
		self.windowDidResize({});
		return self;
	},
	disable: function() {
		var self = this;
		$(self.domEl).css({ display: "none" });
		theTLSettings.removeAsObserver({
			type: "currentDate",
			observer: self
		});
			
	},
	enable: function() {
		var self = this;
		theTLSettings.registerAsObserver({
			type: "currentDate",
			observer: self
		});
		self.currentDateWasUpdatedTo({ date: theTLSettings.currentDate });
		$(self.domEl).css({ display: "block" });
		theTLMainController.updateMarkersImageSize();
	},
	getEquallySpacedOffsetForDate: function(data) {
		var self = this;
		var aDate = data.aDate;
		var numItems = self.items.length;
		var lastFoundItem = false;
		var offset = 0;
		for (var counter = 0; counter < numItems; counter++) {
			var thisItem = self.items[counter];
			if (aDate.getTime() <= thisItem.startDate.getTime()) {
				lastFoundItem = thisItem;
				break
			}
		}
		if(lastFoundItem) {
			var prevItem = self.items[lastFoundItem.orderIndex-1];
			if (prevItem) { // The offset is between lastFoundItem and prevItem
				if (lastFoundItem.startDate.getTime() != prevItem.startDate.getTime()) { // prevent dividing by zero
					var extra = (lastFoundItem.leftOffset - prevItem.leftOffset)*(aDate.getTime()-prevItem.startDate.getTime())/(lastFoundItem.startDate.getTime()-prevItem.startDate.getTime());
				}
				else {
					extra = 0;	
				}
				offset = prevItem.leftOffset + extra;
			}
			else {
				offset = -1;
				// the offset is between the start of the timeline and the first marker
			}	
		}
		else { // The offset is between the last marker and the end of the timeline
			lastFoundItem = self.items[self.items.length-1];
			var viewStageWidth = self.width;
			if (lastFoundItem.startDate.getTime() != self.endDate.getTime()) { // prevent dividing by zero
				var extra = (viewStageWidth - lastFoundItem.leftOffset)*(aDate.getTime()-lastFoundItem.startDate.getTime())/(self.endDate.getTime()-lastFoundItem.startDate.getTime());
			}
			else {
				extra = 0;	
			}
			offset = lastFoundItem.leftOffset + extra;
		}
		return offset;
	},
	createCategoryBands: function() {
		var self = this;
		var spaceRemaining = 100;
		
		var ourCategories = [];
		var totalSize = 0;
		$.each(self.timeline.categories, function() { // Filter out hidden categories and default category if it is empty
			if (!this.hide && !(this.autoGenerated && theTLMainController.markersByCategory[this.key] && theTLMainController.markersByCategory[this.key].markers.length == 0)) {
				ourCategories.push(this);
				totalSize+=parseInt(this.size,10);
			}
		}); 
		var itemsRemaining = ourCategories.length;
		var remainingSize = totalSize;
		self.activeCategories = ourCategories;

		/*if (self.timeline.categoriesByKey["tl-category-key-default"] && !theTLMainController.markersByCategory["tl-category-key-default"]) {
			// if the defaultCategory is included in the list of categories but it contains no stories, we exclude it
			itemsRemaining--;
			remainingSize -= theTLMainController.defaultCategory.size;
		}*/
		if (self.timeline.categories.length == 0) {
			// But if there are no categories, we ensure we at least display the default category
			ourCategories = [theTLMainController.defaultCategory];
			itemsRemaining = 1;
			remainingSize = theTLMainController.defaultCategory.size;
		}
		itemsRemaining = (itemsRemaining) ? itemsRemaining : 1;
		$.each(ourCategories, function() {
			/*if (this.autoGenerated &&  theTLMainController.markersByCategory[this.key] && theTLMainController.markersByCategory[this.key].markers.length == 0) {
				// if default category is empty, don't show its band
				return;	
			}*/
			this.bandClass="";
			var bandPercentHeight = Math.round(parseInt(this.size,10)/remainingSize * spaceRemaining);
			this.percentHeight = bandPercentHeight;
			var bandTop = 100 - spaceRemaining;
			spaceRemaining -= bandPercentHeight;
			remainingSize -= parseInt(this.size,10);
			itemsRemaining--;
			var aCatBand = new TLViewCategoryBand({
				category: this,
				height: bandPercentHeight,
				top: bandTop,
				width: self.width
			}).init();
			self.categoryBands.push(aCatBand);
			self.categoryBandsByKey[this.key]=aCatBand;
			$(self.domEl).append(aCatBand.domRoot);
			$(theTLMainController.domFixedStageContent).append(aCatBand.domLabel);
		});
	},
	displayMarkers: function() {
		var self = this;
		var domDiv = $('<div class="tl-stage-story-holder"></div>').get()[0];
		self.domStageStoryHolder = domDiv; 
		var startTimeNum = self.startDate.getTime();

		// Firstly, work out leftoffset for the items
		$.each(self.items, function() {
			if (self.timeline.markerSpacing == "equal") {
				this.leftOffset = self.timeline.equalMarkerLeftPadding+ (this.orderIndex * self.timeline.equalMarkerSpacing); 
			}
			else {
				this.leftOffset = -self.getLeftOffsetForDate({ date: this.startDate });
			}
		});

		// Now calculate how many markers are within 300px of a marker
		self.generateMarkerSizeAndPositions();
		self.adjustMarkerPositions();
		$.each(self.items, function() {
			var domMarker = this.generateDom();
			if (self.timeline.viewType == "category-band") {
				if (self.categoryBandsByKey[this.category.key]) {
					$(self.categoryBandsByKey[this.category.key].domRoot).append(domMarker);
				}
			}
			else {
				if (self.timeline.viewType == "duration") {
					$(domDiv).append(domMarker);
				}
				else {
					$(domDiv).prepend(domMarker);
				}
			}
		});
		
		if (self.timeline.viewType != "category-band") {
			$(self.domEl).append(domDiv);
		}
		
	},
	destroy: function() {
		var self = this;
		$(self.domStageStoryHolder).remove();
		$(self.domViewBlocksHolder).remove();
		
		$(self.domEl).removeClass("tl-stage-"+self.type);
		$(self.domEl).removeClass("tl-stage-view-"+self.currentViewType);
		if (self.currentViewType == "duration") {
			$(self.domEl).removeClass("tl-stage-view-color-category-stories");
		}
		$(self.domEl).unbind();
		self.draggableObj.kill();
		theTLSettings.removeAsObserver({
			type: "currentDate",
			observer: self
		});

		theAJKWindowResizeEvent.removeAsObserver({ observer: self });

	},
	refreshMarkersAfterFilterChange: function() {
		var self = this;
		self.generateMarkerSizeAndPositions();
		self.refreshDisplayMarkers();
		$.each(self.items, function() {
			this.positionDisplayImageInContainer();
		});
	},
	addNewMarkerToStage: function(data) {
		var self = this;
		var marker = data.marker;
		marker.leftOffset = -self.getLeftOffsetForDate({ date: marker.startDate });
		self.generateMarkerSizeAndPositions();
		var domMarker = marker.generateDom();
		$(self.domStageStoryHolder).prepend(domMarker);
		self.refreshDisplayMarkers();
		
		theTLMainController.updateMarkersImageSize();
	},
	markerWasDeleted: function() {
		var self = this;
		self.generateMarkerSizeAndPositions();
		self.refreshDisplayMarkers();
		theTLMainController.updateMarkersImageSize();
	},
	refreshDisplayMarkers: function() {
		var self = this;
		// Firstly, work out leftoffset for the items
		$.each(self.items, function() {
			this.initialHorizAdjustment = 0;
			this.horizAdjustment = 0;
			this.numCloseMarkers = 0;
			this.sizeClass = "";
			this.vSize = "normal";
			this.leftOffset = -self.getLeftOffsetForDate({ date: this.startDate });
		});
		// Now calculate how many markers are within 300px of a marker
		self.generateMarkerSizeAndPositions();
		self.adjustMarkerPositions();

		$.each(self.items, function() {
			this.refreshPositionAndSize();
		});		
	},
	adjustMarkerPositions: function() {
		var self = this;
		
		
		if (self.timeline.viewType == "duration" || self.timeline.markerSpacing == "top-to-bottom") {
			// Duration view markers are no adjusted horizontally but vertically
			$.each(self.items, function() {
				this.horizAdjustment = 0;
			});
			return;
		}
			
		var itemIndex = 0;
		var verticalPos = 0;
		var maxVerticalPos = 10;
		
		var minIdealStorySeparation = [];
		minIdealStorySeparation["normal"] = 327;
		minIdealStorySeparation["small"] = 327;
		minIdealStorySeparation["very-small"] = 327;
		minIdealStorySeparation["tiny"] = 327;
		minIdealStorySeparation["miniature"] = 175;
		minIdealStorySeparation["category-normal"] = 238;
		minIdealStorySeparation["category-small"] = minIdealStorySeparation["category-small-saved"] = 175;
		
		var minAdjust = [];
		var maxAdjust = [];
		minAdjust["normal"] = -132;
		maxAdjust["normal"] = 130;
		minAdjust["small"] = -132;
		maxAdjust["small"] = 130;
		minAdjust["very-small"] = -132;
		maxAdjust["very-small"] = 130;
		minAdjust["tiny"] = -132;
		maxAdjust["tiny"] = 130;
		minAdjust["miniature"] = -56;
		maxAdjust["miniature"] = 54;
		minAdjust["category-small"] = -56;
		maxAdjust["category-small"] = 54;
		minAdjust["category-normal"] = -86;
		maxAdjust["category-normal"] = 84;

		var getItemOfSimilarVerticalPosition = function(data) {
			var anItem = data.anItem;
			var itemIndex = (self.timeline.viewType == "category-band") ? anItem.categoryIndex : data.itemIndex;
			var verticalPos = data.verticalPos;
			var compareItem = "";
			var direction = data.direction; // either 1 or -1
			var compareList = (self.timeline.viewType == "category-band") ? theTLMainController.markersByCategory[anItem.category.key].markers : self.items;
			
			var compareLimit = 2;
			compareLimit = (anItem.vSize == "normal" || anItem.vSize == "category-normal") ? 10 : compareLimit;
			while (compareItem = compareList[itemIndex+=direction]) {
				if (Math.abs(compareItem.verticalPos-anItem.verticalPos) <= compareLimit || compareItem.vSize == "normal") {
					return compareItem;	
				} 
			}
			return false;
		}
		
		$.each(self.items, function() {
			
			if (self.timeline.viewType == "category-band" && this.category.bandSize != "category-normal") {
				minIdealStorySeparation["category-small"] = minIdealStorySeparation["category-normal"];
			}
			
			var nextItem = getItemOfSimilarVerticalPosition({
				anItem: this,
				itemIndex: itemIndex,
				verticalPos: this.verticalPos,
				direction: 1	
			});
			var previousItem = getItemOfSimilarVerticalPosition({
				anItem: this,
				itemIndex: itemIndex,
				verticalPos: this.verticalPos,
				direction: -1	
			});
			var adjustmentAmount = 0;
			
			var currentItemAdjustedPos = this.leftOffset + this.horizAdjustment;
			var nextItemAdjustedPos = nextItem.leftOffset + nextItem.horizAdjustment;
			var previousItemAdjustedPos = previousItem.leftOffset + previousItem.horizAdjustment;
			var nextItemClose = false;
			var prevItemClose = false;

			if (nextItem && (nextItemAdjustedPos-currentItemAdjustedPos < minIdealStorySeparation[this.vSize])) {
				adjustmentAmount = (nextItemAdjustedPos-currentItemAdjustedPos - minIdealStorySeparation[this.vSize]);  
				adjustmentAmount = (adjustmentAmount < minAdjust[this.vSize]) ? minAdjust[this.vSize] : adjustmentAmount;
				nextItemClose = true;
			}
			else if (previousItem && (currentItemAdjustedPos - previousItemAdjustedPos < minIdealStorySeparation[previousItem.vSize])) {
				adjustmentAmount = (currentItemAdjustedPos - previousItemAdjustedPos + minIdealStorySeparation[previousItem.vSize]);  
				adjustmentAmount = (adjustmentAmount > maxAdjust[this.vSize]) ? maxAdjust[this.vSize] : adjustmentAmount;
				prevItemClose = true;
			}
			
			if (nextItemClose && !prevItemClose) {
				adjustmentAmount = minAdjust[this.vSize];
				if (previousItem) {
					adjustmentAmount = (previousItemAdjustedPos-currentItemAdjustedPos + minIdealStorySeparation[previousItem.vSize]);  
					adjustmentAmount = (adjustmentAmount < minAdjust[this.vSize]) ? minAdjust[this.vSize] : adjustmentAmount;
					adjustmentAmount = (adjustmentAmount > maxAdjust[this.vSize]) ? maxAdjust[this.vSize] : adjustmentAmount;
				}
			}
			
			/*if (prevItemClose && !nextItemClose) {
				adjustmentAmount = maxAdjust;
				if (nextItem) {
					adjustmentAmount = (nextItemAdjustedPos - currentItemAdjustedPos - minIdealStorySeparation);  
					alert(this.headline+" "+adjustmentAmount);
					adjustmentAmount = (adjustmentAmount > maxAdjust) ? maxAdjust : adjustmentAmount;
				}
			}*/
			this.horizAdjustment = adjustmentAmount; 
			
			//alert(this.horizAdjustment);

			itemIndex++;

			minIdealStorySeparation["category-small"] = minIdealStorySeparation["category-small-saved"];
		});

	},
	calculateTopToBottomPositions: function(data) {
		var self = this;
		var numRows = data.numRows;
		var maxVerticalPos = data.maxVerticalPos;
		var rowCounter = numRows-1; 
		var items = data.items;
		var rowLastDate = [];
		var bottomToTop = (data.bottomToTop) ? data.bottomToTop : false;

		$.each(items, function() {
			if (self.timeline.viewType == "duration" && this.startDate.getTime() < this.endDate.getTime()) {
				// Duration story so need to check if the current row has space for it
				// If not, we place it in the next row that has space
				var currentRow = rowCounter, inc = 0;
				while (rowLastDate[currentRow] && rowLastDate[currentRow] > this.startDate.getTime() && inc++ < numRows) {
					if (--currentRow < 0) {
						currentRow = numRows-1;
					}
				}
				if (inc == numRows) {
					// Couldn't find a free row so leave it in currentRow;
					this.setVerticalPos({pos: (bottomToTop) ? (numRows-1-rowCounter)/numRows * maxVerticalPos : rowCounter/numRows * maxVerticalPos });
					rowLastDate[rowCounter] = this.endDate.getTime();
				}
				else {
					// Found a free row so put it in the free row
					this.setVerticalPos({pos: (bottomToTop) ? (numRows-1-currentRow)/numRows * maxVerticalPos : currentRow/numRows * maxVerticalPos });
					rowLastDate[currentRow] = this.endDate.getTime();
					if (currentRow != rowCounter) {
						rowCounter++;
					}
				}
			}
			else { 
				this.setVerticalPos({pos: (bottomToTop) ? (numRows-1-rowCounter)/numRows * maxVerticalPos : rowCounter/numRows * maxVerticalPos });
			}
			
			if (self.timeline.viewType == "duration" || (self.timeline.viewType == "category-band" && this.category.viewType == "duration")) {
				this.generateDurationSizeData();
			}
			else {
				this.generateSizeData();
			}
			if (--rowCounter < 0) {
				rowCounter = numRows-1;
			}
		});
	},
	calculateStandardSpacingPositions: function(data) {
		var self = this;
		var items = data.items;
		var itemIndex = 0;
		var verticalPos = 0;
		var maxVerticalPos = 10;

		$.each(items, function() {
			var compareIndex = (self.timeline.viewType == "category-band") ? this.categoryIndex+1 : itemIndex+1;
			var compareList = (self.timeline.viewType == "category-band") ? theTLMainController.markersByCategory[this.category.key].markers : self.items;
			while (compareList[compareIndex] && (compareList[compareIndex].leftOffset + compareList[compareIndex].horizAdjustment) - (this.leftOffset + this.horizAdjustment) < self.minIdealStorySeparation) {
				this.numCloseMarkers++;
				compareList[compareIndex].numCloseMarkers++;
				compareIndex++;
			}
			this.generateSizeData();
			if (self.timeline.viewType == "category-band") {
				if (this.vSize == "category-normal") { 
					verticalPos = 0;
					this.setVerticalPos({pos: 0 });
				}
				else {
					verticalPos = (verticalPos > 6) ? 0 : verticalPos;
					this.setVerticalPos({pos: verticalPos });				
					verticalPos += 4.2;
				}
			}
			else if ( this.vSize == "normal") {
				verticalPos = 0;
				this.setVerticalPos({pos: 0 });
			}
			else if (this.vSize == "small") {
				verticalPos = (verticalPos > 6) ? 0 : verticalPos;
				this.setVerticalPos({pos: verticalPos });				
				verticalPos += 5;
			}
			else if (this.vSize == "very-small") {
				verticalPos = (verticalPos > 7) ? 0 : verticalPos;
				this.setVerticalPos({pos: verticalPos });				
				verticalPos += 4;
			}
			else if (this.vSize == "tiny") {
				verticalPos = (verticalPos > 7.5) ? 0 : verticalPos;
				this.setVerticalPos({pos: verticalPos });				
				verticalPos += 2.5;
			}
			else if (this.vSize == "miniature") {
				verticalPos = (verticalPos > 7.5) ? 0 : verticalPos;
				this.setVerticalPos({pos: verticalPos });				
				verticalPos += 2.5;
			}
			itemIndex++;
		});

	},
	generateMarkerSizeAndPositions: function() {
		var self = this;	
		
		$.each(self.items, function() {
			this.numCloseMarkers=0;
		});

		if ((self.timeline.viewType != "category-band") && (self.timeline.viewType == "duration" || self.timeline.markerSpacing == "top-to-bottom")) {
			self.calculateTopToBottomPositions({
				numRows: (self.timeline.viewType == "duration" && self.timeline.markerSpacing != "top-to-bottom") ? 5 : self.timeline.markerSpacingObj.rows,
				maxVerticalPos: (self.timeline.viewType == "duration") ? 10 : 9.2,
				items: self.items
			});
		}
		else if (self.timeline.viewType == "category-band") {
			$.each(self.timeline.categories, function() {
				if (this.viewType == "duration") {
					self.calculateTopToBottomPositions({
						numRows: this.rows,
						maxVerticalPos: 9,
						items: theTLMainController.markersByCategory[this.key].markers,
						bottomToTop: true
					});
				}
				else {
					self.calculateStandardSpacingPositions({
						items: theTLMainController.markersByCategory[this.key].markers
					});
				}
				
			});	
		}
		else {
			self.calculateStandardSpacingPositions({
				items: self.items
			});
		}
	},
	generateMinMaxStagePositionsFromVisibleDates: function() {
		var self = this;
		self.minStagePos = self.getCentredLeftOffsetForDate({ date: theTLSettings.visibleStartDate});	
		self.maxStagePos = self.getCentredLeftOffsetForDate({ date: theTLSettings.visibleEndDate});	
		//alert(self.minStagePos+" "+self.maxStagePos)
	},
	initialiseViewDrag: function() {
		var self = this;
		self.draggableObj = new AJKDraggable({
			domDragEl: self.domEl,
			limitFunc: function() {
				var limit = {
					minX: self.minStagePos,
					maxX: self.maxStagePos,
					minY: 0,
					maxY: 0
				}
				return limit;			
			},
			mouseMoveFunc: function(data) {
				var leftOffset = -data.dragElPos.x;
				self.stageLeftOffset = - leftOffset;
				self.realStageLeftOffset = self.stageLeftOffset;
				var fraction = (leftOffset+0.5*theTLSettings.windowSize.width)/self.width;
				var aDate = AJKHelpers.getEmptyDate();
				aDate.setTime(self.startDate.getTime()+fraction*self.dateRange);
				theTLSettings.setCurrentDate({ 
					date: aDate,
					extraInfo: {
						drag: true						
					}
				 });
			},
			dragDidStartFunc: function() {
				theTLSettings.lastSelectedStory = "";	
			},
			dragDidEndFunc: function() {
				// Need to update hash to currentTime
				theTLHashController.setHashToCurrentDate();
			}
		}).init();
				
	},
	createViewScaleBlocks: function() {
		var self = this;

		var aDiv = $("<div></div>").get()[0];
		self.domViewBlocksHolder = aDiv;
		
		if (self.timeline.markerSpacing == "equal") {
			var insertHTML = '<div style="background: url(assets/ui/stage/zebra-'+self.timeline.equalMarkerSpacing+'px.png) '+parseInt(self.timeline.equalMarkerLeftPadding-0.5 * self.timeline.equalMarkerSpacing,10)+'px 0; width: '+self.width+'px;" class="scale-block">';
			insertHTML += '<div style="background: url(assets/ui/stage/standard-scale-'+self.timeline.equalMarkerSpacing+'px-segments.png) '+parseInt(self.timeline.equalMarkerLeftPadding-0.5 * self.timeline.equalMarkerSpacing,10)+'px bottom repeat-x;" class="index">';
			for (var counter = 0; counter < self.items.length; counter++) {
				var thisMarker = self.items[counter];
				var offset = self.timeline.equalMarkerLeftPadding + (counter * self.timeline.equalMarkerSpacing);
				insertHTML += '<div style="left: '+offset+'px" class="scale-block-label"><span id="tl-marker-equal-spacing-date-displayer-'+thisMarker.id+'">'+thisMarker.formatDisplayDate({ startDateOnly: true })+'</span></div>'
			}
			
			insertHTML += '</div>';
			insertHTML += '<div class="content"></div>';
			insertHTML += '</div>';

			$(aDiv).append(insertHTML);

		}
		else { // Default view
			var thisBlockStartDate = TLViewController.viewScaleSettings[self.type].getFirstBlockStartDateFromDate({date: self.startDate});
			var counter = 0;
			
			while (thisBlockStartDate.getTime() < self.endDate.getTime() && counter<5000) {
				var thisBlockDateRange = TLViewController.viewScaleSettings[self.type].getDateRangeInMS({ blockStartDate: thisBlockStartDate });
				
				var aViewScaleBlock = new TLViewScaleBlock({
					type: self.type,
					leftOffset: - self.getLeftOffsetForDate({ date: thisBlockStartDate }),
					width: self.getWidthOfDateRange({ dateRange: thisBlockDateRange }),
					text: TLViewController.viewScaleSettings[self.type].getTextForDate({ date: thisBlockStartDate }),
					colour: self.scaleColour,
					viewScale: TLViewController.viewScaleSettings[self.type],
					controller: self,
					index: counter
				}).init();
	
				if (TLViewController.viewScaleSettings[self.type].adjustDomBlock) {
					TLViewController.viewScaleSettings[self.type].adjustDomBlock({ domBlock: aViewScaleBlock.domEl, date: thisBlockStartDate }); 	
				}
				self.viewScaleBlocks.push(aViewScaleBlock);
				$(aDiv).append(aViewScaleBlock.domEl);
				thisBlockStartDate.setTime(thisBlockStartDate.getTime() + thisBlockDateRange);
				counter++;
			}
		}
		
		$(self.domEl).css({ width: self.width }).append(aDiv);

	},
	currentDateWasUpdatedTo: function(data) {
		var self = this;
		var currentDate = data.date;
		var animate = (data.extraInfo) ? data.extraInfo.animate : false;
		if (animate) {
			self.animateViewToDate({date: currentDate, speed: theTLSettings.animateTime });
		}
		else if (data.extraInfo && data.extraInfo.viewOnlySpeed) {
			self.animateViewToDate({date: currentDate, speed: data.extraInfo.viewOnlySpeed });
		}
		else {	
			self.slideViewToDate({date: currentDate, instantly: !animate });
		}
		self.updateDisplayDate({ date: currentDate });
	},
	getClosestMarkerIndexToRealDate: function(data) {
		var self = this;
		var testDateTime = data.date.getTime();
		var smallestDiff = -1;
		var lastMarker = false;
		var selectedMarker = false;
		
		var counter = 0;
		while (self.items[counter]) {
			var thisMarker = self.items[counter];
			var diff = Math.abs(thisMarker.startDate.getTime()-testDateTime);
			if (diff > smallestDiff && !selectedMarker && lastMarker) {
				selectedMarker = lastMarker;
			}
			lastMarker = thisMarker;
			smallestDiff = diff;
			counter++;
		}
		
		return selectedMarker || lastMarker;
	},
	getClosestMarkerIndexToDate: function(data) {
		var self = this;
		var aDate = data.date;
		var centreStagePos = (aDate.getTime() - self.startDate.getTime())/(self.endDate.getTime() - self.startDate.getTime())*self.width + 0.5 * self.timeline.equalMarkerSpacing;
		var nearestMarker = parseInt((centreStagePos - self.timeline.equalMarkerLeftPadding)/self.timeline.equalMarkerSpacing,10);
		return nearestMarker;
	},
	getEqualSpacingDateFromStandardDate: function(data) {
		var self = this;
		var aDate = data.aDate;
		var nearestMarker = self.getClosestMarkerIndexToDate({date: aDate });
		if (nearestMarker < 0) {
			aDate = self.startDate;
		}
		else if (self.items[nearestMarker]) {
			aDate = self.items[nearestMarker].startDate;
		}
		else {
			aDate = self.endDate;
		}
		return aDate;
	},
	updateDisplayDate: function(data) {
		var self = this;
		var aDate = data.date;
		var format = TLViewController.viewScaleSettings[self.type].displayDateFormat;
		format = (!format) ? "ddnn MMMM YYYY HH:00" : format; 	
		format = (self.timeline.topDateFormat != "auto" ) ? self.timeline.topDateFormat : format;
		
		// if we are in equal spacing mode, we need to calculate the display date another way.
		if (self.timeline.markerSpacing == "equal") {
			aDate = self.getEqualSpacingDateFromStandardDate({aDate: aDate });			
		}

		var displayDate = AJKHelpers.formatDate({
			date: aDate,
			formatString: format
		});
		
		if (displayDate != self.displayDate) {
			$(self.domDateDisplayer).text(displayDate);
			self.displayDate = displayDate;
		}
		if (self.displayDate) {
			$(self.domDateDisplayer).css({ display: "block" });
		}
		else {
			$(self.domDateDisplayer).css({ display: "none" });
		}
	},
	slideViewToDate: function(data) {
		var self = this;
		var date = data.date;
		var instantly = data.instantly;
		self.stageLeftOffset = self.getCentredLeftOffsetForDate({date: date});
		if ((!self.realStageLeftOffset && self.realStageLeftOffset !== 0) || instantly) {
			self.realStageLeftOffset = self.stageLeftOffset; 
		} 
		self.startStageEasing();

		//$(self.domEl).css({ left: self.stageLeftOffset });
	},
	animateViewToDate: function(data) {
		var self = this;
		var date = data.date;
		var speed = (data.speed) ? data.speed : 500;
		self.currentlyAnimating = true;	
		self.stageLeftOffset = self.getCentredLeftOffsetForDate({date: date});
		self.realStageLeftOffset = self.stageLeftOffset; 
		$(self.domEl).stop().animate({ left: self.stageLeftOffset },speed, function() { self.currentlyAnimating=false });

	},
	getCentredLeftOffsetForDate: function(data) {
		var self = this;
		var date = data.date;
		var offset = (date.getTime()-self.startDate.getTime())/self.dateRange * self.width;
		offset = offset - 0.5 * theTLSettings.windowSize.width;	
		return -offset;
	},
	getLeftOffsetForDate: function(data) {
		var self = this;
		var date = data.date;
		var offset = (date.getTime()-self.startDate.getTime())/self.dateRange * self.width;
		return -Math.round(offset);
	},
	getDateFromLeftOffset: function(data) {
		var self = this;
		var leftOffset = data.leftOffset;
		var dateInMS = Math.round((leftOffset *self.dateRange /   self.width) + self.startDate.getTime());
		var aDate = new Date();
		aDate.setTime(dateInMS);
		return aDate;		
	},
	getWidthOfDateRange: function(data) {
		var self = this;
		var dateRange = data.dateRange;
		return Math.round(self.width * dateRange/self.dateRange);
	},
	startStageEasing: function() {
		var self = this;	

		if (!self.animationTimeout) {
			(function() {
				if (self.currentlyAnimating) {
					return;	
				}
				//alert(self.realStageLeftOffset+" "+);
				self.realStageLeftOffset += (self.stageLeftOffset - self.realStageLeftOffset)/self.easingFraction;
				$(self.domEl).css({ left: self.realStageLeftOffset });
				if (parseInt(self.stageLeftOffset-self.realStageLeftOffset,10) != 0) {
					var thisFunc = arguments.callee;
					self.animationTimeout = setTimeout(function() {
						thisFunc();
					},self.easeTimeoutTime);
				} 
				else {
					self.animationTimeout = "";
				}				
			})();	
		}
	},
	windowDidResize: function(data) {
		var self = this;
		setTimeout(function() { // Need to wait until after the slider has recalibrated
			self.slideViewToDate({ date: theTLSettings.currentDate, instantly: true });
			self.generateMinMaxStagePositionsFromVisibleDates();
		},1);
	},
	updateScaleColour: function(data) {
		var self = this;
		var colour = data.colour;
		$.each(self.viewScaleBlocks, function() {
			this.updateScaleColour(data);
		});

	}
}

var TLViewCategoryBand = function(data) {
	var self = this;
	self.category = data.category;
	self.height = data.height;
	self.top = data.top;
	self.width = data.width;
	self.domRoot = "";
	self.domLabel = "";
}

TLViewCategoryBand.prototype = {
	init: function() {
		var self = this;
		self.generateDom();
		return self;
	},
	generateDom: function() {
		var self = this;
		var highlightColor = ($.browser.msie) ? AJKHelpers.adjustColour({colour: self.category.colour, adjust: 1.2 }) : self.category.colour;
		var extraClass = (self.category.viewType == "duration") ? " tl-stage-view-duration tl-stage-view-color-category-stories" : "";
		var insertHTML = '<div class="tl-view-category-band'+extraClass+'" style="width: '+self.width+'px; border-color: #'+highlightColor+'; height: '+self.height+'%; top: '+self.top+'%;">';
		insertHTML += '<div class="tl-vcb-scale" style="background-color: #'+self.category.colour+';"></div>';
		insertHTML += '<div style="background-color: #'+self.category.colour+';" class="tl-vcb-inner"></div>';
		insertHTML += '</div>';
		self.domRoot = $(insertHTML).get()[0];
		var categoryTitle = AJKHelpers.clipToMaxCharWords({
			aString: self.category.title,
			maxChars: 30
		});
		
		if ($.browser.msie) {
			var ieWidth = 30000;
			while (ieWidth < self.width) {
				$(self.domRoot).append('<div style="left: '+ieWidth+'px; background-color: #'+self.category.colour+';" class="tl-vcb-inner"></div>');
				ieWidth+=30000;
			}
		}
						
		var insertHTML = '<div class="tl-view-category-band-label" style="top: '+(self.height+self.top)+'%;"><h4 style="background-color: #'+highlightColor+';">'+categoryTitle+'</h4></div>';
		self.domLabel = $(insertHTML).get()[0];
	},
	refreshFromCategory: function() {
		var self = this;
		$(self.domLabel).find("h4").css({ backgroundColor: "#"+self.category.colour }).text(self.category.title);	
		$(self.domRoot).css({ borderColor: "#"+self.category.colour });	
		$(self.domRoot).find(".tl-vcb-inner, tl-vcb-scale").css({ backgroundColor: "#"+self.category.colour });	
	}
}

var TLViewScaleBlock = function(data) {
	var self = this;
	self.type = data.type;
	self.viewScale = data.viewScale;
	self.numItems = (self.viewScale.numItems) ? self.viewScale.numItems : 1; 
	self.index = data.index;
	self.domEl = "";
	self.width = data.width;
	self.text = data.text;
	self.leftOffset = data.leftOffset;
	self.colour = data.colour;
	self.domColourBlock;
	self.itemWidth = self.width/self.numItems;
	self.controller = data.controller;
	self.scaleGraphic = self.viewScale.segmentWidth ? 'assets/ui/stage/standard-scale-'+self.viewScale.segmentWidth+'px-segments.png' : 'assets/ui/stage/scale-'+self.type+'.png';
	self.foreignScaleGraphic = self.viewScale.segmentWidth ? 'assets/ui/stage/standard-scale-'+self.viewScale.segmentWidth+'px-segments.png' : 'assets/ui/stage/foreign/scale-'+self.type+'.png';
}

TLViewScaleBlock.prototype = {
	init: function() {
		var self = this;
		var viewScaleExtraClass = " scale-block-"+self.type;
		var lang = theTLMainController.timeline.language;
		var scaleGraphic = (lang && lang != "english" && lang != "english-common") ? self.foreignScaleGraphic : self.scaleGraphic;
		var posClass = (self.index % 2) ? " scale-block-odd" : "";
		var blockStyle = (self.viewScale.segmentWidth) ? 'style="background: url(assets/ui/stage/zebra-'+self.viewScale.segmentWidth+'px.png)"' : "";
		var insertHTML = '<div '+blockStyle+'class="scale-block '+viewScaleExtraClass+posClass+'">';
		var backgroundStyle = (self.viewScale.segmentWidth) ? "none" : 'url('+scaleGraphic+') left bottom repeat-x';
		insertHTML += '<div style="background: '+backgroundStyle+';" class="index"><h4>'+self.text+'</h4>';
		for (var counter = 1; counter < self.numItems; counter++) {
			var offset = counter * self.itemWidth;
			var itemPos = Math.round(offset + 0.5 * self.itemWidth);
			var itemDate = self.controller.getDateFromLeftOffset({ leftOffset: self.leftOffset  + offset + 0.0005 * self.itemWidth});
			if (self.viewScale.segmentWidth) {
				insertHTML += '<div style="left: '+(counter*self.viewScale.segmentWidth)+'px; width: '+(self.viewScale.segmentWidth-2)+'px;" class="scale-block-label-v2"><span>'+self.viewScale.getTextForDate({ date: itemDate, subItem: true })+'</span></div>'
			}
			else {
				insertHTML += '<div style="left: '+itemPos+'px" class="scale-block-label"><span>'+self.viewScale.getTextForDate({ date: itemDate, subItem: true })+'</span></div>'
			}
		}
		
		insertHTML += '</div>';
		insertHTML += '<div class="content"></div>';
		insertHTML += '</div>';
		self.domEl = $(insertHTML).get()[0];
		$(self.domEl).css({width: self.width, left: self.leftOffset });
		self.domColourBlock = $(self.domEl).find("h4").get()[0]; 
		$(self.domColourBlock).css({ background: "#"+self.colour });
		if (self.numItems > 1 && self.viewScale.segmentWidth) {
			$(self.domColourBlock).css({ left: 2, width: self.viewScale.segmentWidth-4 });
		}
		if (self.viewScale.segmentWidth && (self.numItems % 2) && (self.index % 2)) {
			$(self.domEl).css({ backgroundPosition: -Math.max(self.itemWidth)+"px bottom" });
		}

		return self;
	},
	setLeftPosition: function(data) {
		var self = this;
		var position = data.position;
		$(self.domEl).css({ left: position });
	},
	updateScaleColour: function(data) {
		var self = this;	
		self.colour = data.colour;
		$(self.domColourBlock).css({ background: "#"+self.colour });
	}
}
